@@ -27,48 +27,32 @@ from .digital import daqdownsamplingfilters
 
 
 class FIRfilter(object):
-    """ Model parameters for FIR filter creation
-    Parameters.
+    """
+    Model parameters for FIR filter creation
+
+    Parameters
     ----------
-    fNyquist : int
+    fNyq : `int`, optional
         Nyquist frequency of FIR filter
-    dur : float
+    dur : `float`, optional
         Duration in seconds of FIR filter
-    latency : float
+    highpass_fcut : `float`, optional
+        Cut off frequency for high-pass filter
+    lowpass_fcut : `float`, optional
+        Cut off frequency for low-pass filter (relevant for inverse sensing filter)
+    latency : `float`, optional
         Latency in seconds of FIR filter
-    df : float
-        Frequency spacing of the FIR filter
-    window_type : str
+    window_type : `str`, optional
         Type of window function to use. Options are 'dpss', 'kaiser',
         'dolph_chebyshev', and 'hann'.
-    window : float
-        Window function, stored as an array
-    freq_res: float
+    freq_res: `float`, optional
         Frequency resolution of the FIR filter, computed as half the width of
-        the central lobe.
-    dt : float
-        Inverse of the sampling rate
-    freq_array : float
-        Frequency array from 0 Hz up to and including Nyquist
-    highpass_fcut : float
-        Cut off frequency for high-pass filter
-    lowpass_fcut : float
-        Cut off frequency for low-pass filter (relevant for inverse sensing filter)
-    samples_to_HPcorner : int
-        Number of samples in the frequency domain below the high-pass corner frequency
-    samples_to_LPcorner : int
-        Number of samples in the frequency domain above the low-pass corner frequency
-    delay_samples : int
-        Number of samples to delay filter.  This is related to the desired latency.
-    delay_array : complex
-        complex array of the delay to be applied at each frequency
-    advance_array : complex
-        complex array of the advance to be applied at each frequency required to
-        undo the artificial delay
+
     Returns
-    ----------
+    -------
     FIRfilter : list
         List of model parameters for FIR filter creation parameters
+
     """
     def __init__(self, fNyq=8192, dur=0, highpass_fcut=10, lowpass_fcut=None,
                  latency=None, window_type='dpss', freq_res=3.0):
@@ -84,8 +68,8 @@ class FIRfilter(object):
 
         if dur:
             if window_type == 'hann':
-                warnings.warn("Frequency resolution can not be set "
-                              "since you are using Hann window.")
+                warnings.warn("Frequency resolution can not be set independently "
+                              "of duration since you are using Hann window.")
                 self.freq_res = 2.0 / self.dur
             else:
                 self.freq_res = freq_res
@@ -160,14 +144,14 @@ class FIRfilter(object):
 
 
 def correctFIRfilter(FIRparams, tdfilt, fdmodel, window_correction_range, save_to_file=None):
-    """ Correct FIR filter production errors induced by windowing,
+    """
+    Correct FIR filter production errors induced by windowing,
     and return a compensated version of the frequency-domain model.
 
     Parameters
     ----------
     FIRparams : FIRfilter parameters list
-        a list of model parameters for FIR filter creation
-    parameters
+        a list of model parameters for FIR filter creation parameters
     tdfilt : array-like
         Time-domain model FIR filter
     fdmodel : array-like
@@ -178,9 +162,10 @@ def correctFIRfilter(FIRparams, tdfilt, fdmodel, window_correction_range, save_t
         Filename (NPZ) to save the data from this result
 
     Returns
-    ----------
+    -------
     compensated_model : array-like
         A compensated version of the frequency-domain model
+
     """
     if len(window_correction_range) % 4:
         raise ValueError("correctFIRfilter: Invalid input argument for window_correction_range."
@@ -248,7 +233,8 @@ def correctFIRfilter(FIRparams, tdfilt, fdmodel, window_correction_range, save_t
 
 
 def createFIRfilter(FIRparams, static_model, save_to_file=None):
-    """ Generate an FIR filter based on provided frequency-domain model
+    """
+    Generate an FIR filter based on provided frequency-domain model
 
     Parameters
     ----------
@@ -260,14 +246,15 @@ def createFIRfilter(FIRparams, static_model, save_to_file=None):
         Filename (NPZ) to save the data from this result
 
     Returns
-    ----------
+    -------
     model_FIR : `float64`, array-like
         A time domain FIR filter model
     double_model : `float`, array-like
         An array of real doubles FIR filter model
+
     """
     model = []
-    for i in range(0, len(static_model)):
+    for i in range(len(static_model)):
         model.append(static_model[i])
     model = np.asarray(model, dtype=np.complex128)
 
@@ -288,7 +275,7 @@ def createFIRfilter(FIRparams, static_model, save_to_file=None):
         hp_hann = np.concatenate((np.zeros(16 * FIRparams.samples_to_HPcorner
                                            - len(hp_hann)), hp_hann))
 
-        model[: 16 * FIRparams.samples_to_HPcorner] *= hp_hann
+        model[:16 * FIRparams.samples_to_HPcorner] *= hp_hann
 
     if FIRparams.lowpass_fcut is not None:
         # Create a low-pass filter to be convolved with the FIR model filter
@@ -305,7 +292,7 @@ def createFIRfilter(FIRparams, static_model, save_to_file=None):
     # Add delay to model to make sure it falls off at the edges
     model_FIR = np.roll(model_FIR, int(FIRparams.latency / FIRparams.dt))[:len(model_FIR) // 16]
     freq_array = np.fft.rfftfreq(len(model_FIR), d=FIRparams.dt)
-    if not (freq_array == FIRparams.freq_array).all:
+    if not np.array_equal(freq_array, FIRparams.freq_array):
         print("Frequency arrays for FIR and model don't agree!")
         print("Freq array from FIR filter is ", freq_array)
         print("Freq array from model is ", FIRparams.freq_array)
@@ -339,6 +326,7 @@ def two_tap_zero_filter_response(zeros, sr, freq):
     Returns
     -------
     two_tap : array
+
     """
     filt = np.ones(1)
     two_tap_filt = np.zeros(2)
@@ -377,6 +365,7 @@ def check_td_vs_fd(tdfilt, fdmodel, fNyq=None, delay_samples=None, highpasstd=No
     Returns
     -------
     plot figure
+
     """
     # Check if highpass exists
     if highpasstd is not None:
@@ -449,8 +438,10 @@ def check_td_vs_fd(tdfilt, fdmodel, fNyq=None, delay_samples=None, highpasstd=No
         # Check the frequency-domain model to see if it is sampled correctly
         Nfdmodel = len(fdmodel)
         if Nfdmodel != Nf:
-            warnings.warn("frequency spacing of frequency-domain model is incorrect."
-                          " Resampling - some information may be lost.")
+            # FIXME: The warning below goes off because freqresp intentionally returns
+            # something at a higher sampling frequency than requested.
+            # warnings.warn("frequency spacing of frequency-domain model is incorrect."
+            #              " Resampling - some information may be lost.")
             long_fdmodel = resample(fdmodel, Nf)
         else:
             long_fdmodel = fdmodel
@@ -532,12 +523,11 @@ def check_td_vs_fd_response(invsens_filt, invsens_highpass, TST_filt, PUM_filt,
     filename : str
     plot_title : str
     legend : str, array-like
-    samples_per_lobe : int
-    ymax_increase : int
 
     Returns
     -------
     plot figure
+
     """
     if invsens_highpass is None:
         invsens_highpass = []
@@ -561,6 +551,8 @@ def check_td_vs_fd_response(invsens_filt, invsens_highpass, TST_filt, PUM_filt,
     invsens_highpass_fd = []
     if any(invsens_highpass):
         invsens_highpass_fd = freqresp(invsens_highpass, delay_samples=invsens_highpass_delay)
+    else:
+        invsens_highpass_fNyq = 0.0
 
     TST_fd = freqresp(TST_filt, delay_samples=act_delay)
     PUM_fd = freqresp(PUM_filt, delay_samples=act_delay)
@@ -577,10 +569,10 @@ def check_td_vs_fd_response(invsens_filt, invsens_highpass, TST_filt, PUM_filt,
                [len(invsens_fd), len(invsens_highpass_fd), len(TST_fd), len(PUM_fd), len(UIM_fd),
                 len(act_highpass_fd), len(D), len(R)]]
     # Find the maximum Nyquist frequency and pad anything with a lower Nyquist frequency with zeros.
-    max_fNqy = max(fd_list[1])
+    max_fNyq = max(fd_list[1])
     for i in range(len(fd_list[0])):
-        if fd_list[1][i] < max_fNqy and any(fd_list[0][i]):
-            length_needed = 1 + int(round(float(fd_list[2][i] - 1) * max_fNqy / fd_list[1][i]))
+        if fd_list[1][i] < max_fNyq and any(fd_list[0][i]):
+            length_needed = 1 + int(round(float(fd_list[2][i] - 1) * max_fNyq / fd_list[1][i]))
             if i == 1:
                 # The additional inverse sensing filter should be padded with ones instead.
                 fd_list[0][i] = np.concatenate((fd_list[0][i],
@@ -607,8 +599,8 @@ def check_td_vs_fd_response(invsens_filt, invsens_highpass, TST_filt, PUM_filt,
             fd_list[0][i] = resampled
 
     # Compute a frequency vector
-    df = float(max_fNqy) / (max_length - 1)
-    frequency = np.arange(0, max_fNqy + df, df)
+    df = float(max_fNyq) / (max_length - 1)
+    frequency = np.arange(0, max_fNyq + df, df)
 
     # Add in the high-pass filters' frequency response.
     if any(fd_list[0][1]):
@@ -638,7 +630,14 @@ def check_td_vs_fd_response(invsens_filt, invsens_highpass, TST_filt, PUM_filt,
     for i in range(len(filter_phase)):
         filter_phase[i] = np.angle(filter_response[i]) * 180 / np.pi
 
-    ratio = filter_response / model_response
+    ratio = []
+    assert len(model_response) == len(filter_response)
+    for i in range(len(model_response)):
+        if model_response[i] != 0.0:
+            ratio.append(filter_response[i] / model_response[i])
+        else:
+            ratio.append(0.0)
+    ratio = np.asarray(ratio)
     ratio_mag = abs(ratio)
     ratio_phase = np.zeros(len(ratio))
     for i in range(len(ratio_phase)):
@@ -735,6 +734,7 @@ class FilterGeneration(DARMModel):
             uim lines are periodically injected to measure and
             correct systematic error.
             freqs[0]**np.arange(freqs[1],freqs[2])+freqs[3])
+
         """
         super().__init__(config)
         if 'FIR' in self._config:
@@ -852,25 +852,25 @@ class FilterGeneration(DARMModel):
 
         if hasattr(self, 'ka_pcal_line_freq') and self.ka_pcal_line_freq == '':
             self.ka_pcal_line_freq = 1.0
-            print("EPICS involving the k_a PCAL line freq will be nonsense.")
+            # print("EPICS involving the k_a PCAL line freq will be nonsense.")
         if hasattr(self, 'uim_act_line_freq') and self.uim_act_line_freq == '':
             self.uim_act_line_freq = 1.0
-            print("EPICS involving the UIM line freq will be nonsense.")
+            # print("EPICS involving the UIM line freq will be nonsense.")
         if hasattr(self, 'pum_act_line_freq') and self.pum_act_line_freq == '':
             self.pum_act_line_freq = 1.0
-            print("EPICS involving the PUM line freq will be nonsense.")
+            # print("EPICS involving the PUM line freq will be nonsense.")
         if hasattr(self, 'ktst_esd_line_freq') and self.ktst_esd_line_freq == '':
             self.ktst_esd_line_freq = 1.0
-            print("EPICS involving the TST line freq will be nonsense.")
+            # print("EPICS involving the TST line freq will be nonsense.")
         if hasattr(self, 'kc_pcal_line_freq') and self.kc_pcal_line_freq == '':
             self.kc_pcal_line_freq = 1.0
-            print("EPICS involving the k_c PCAL line freq will be nonsense.")
+            # print("EPICS involving the k_c PCAL line freq will be nonsense.")
         if hasattr(self, 'src_pcal_line_freq') and self.src_pcal_line_freq == '':
             self.src_pcal_line_freq = 1.0
-            print("EPICS involving the SRC PCAL line freq will be nonsense.")
+            # print("EPICS involving the SRC PCAL line freq will be nonsense.")
         if hasattr(self, 'roaming_pcal_line_freq') and self.roaming_pcal_line_freq == '':
             self.roaming_pcal_line_freq = 1.0
-            print("EPICS involving the roaming PCAL line freq will be nonsense.")
+            # print("EPICS involving the roaming PCAL line freq will be nonsense.")
 
         self.coupled_cavity = self.darm.sensing.optical_response(self.fcc, self.fs,
                                                                  self.srcQ,
@@ -881,7 +881,8 @@ class FilterGeneration(DARMModel):
 
     def GDS(self, ctrl_window_type='dpss', res_window_type='dpss', make_plot=True,
             output_filename='GDS.npz', plots_directory='../examples/GDS_plots'):
-        """ GDS FIR filter generation
+        """
+        GDS FIR filter generation
 
         Parameters
         ----------
@@ -899,6 +900,7 @@ class FilterGeneration(DARMModel):
         Returns
         -------
         output_filename.npz, diagnostic plots of filters
+
         """
 
         # Additional zeros necessary to correct the front end output
@@ -1496,7 +1498,8 @@ class FilterGeneration(DARMModel):
 
     def DCS(self, act_window_type='dpss', invsens_window_type='dpss', make_plot=True,
             output_filename='DCS.npz', plots_directory='../examples/DCS_plots'):
-        """ DCS FIR filter generation
+        """
+        DCS FIR filter generation
 
         Parameters
         ----------
@@ -1514,6 +1517,7 @@ class FilterGeneration(DARMModel):
         Returns
         -------
         output_filename.npz, diagnostic plots of filters
+
         """
         # Compute FIR filter parameters for actuation and inverse sensing
         A_FIRpars = FIRfilter(fNyq=self.FIR.act_fnyq,
@@ -1848,4 +1852,3 @@ class FilterGeneration(DARMModel):
                         ktst_esd_line_freq=self.ktst_esd_line_freq,
                         high_pcal_line_freq=self.high_pcal_line_freq,
                         roaming_pcal_line_freq=self.roaming_pcal_line_freq)
-# %%
