@@ -40,6 +40,7 @@ def find_prime_factors(N):
     Returns
     -------
     prime_factors : array-like
+
     """
     prime_factors = np.array([], dtype=int)
     product = N
@@ -66,7 +67,8 @@ def find_M(M_min):
 
     Returns
     -------
-    int(M ~), prime_factors : array-like
+    tuple : int of M and prime factors
+
     """
     M = pow(2, int(np.ceil(np.log2(M_min))))
     prime_factors = 2 * np.ones(int(np.log2(M) + 1), dtype=np.int64)
@@ -109,6 +111,7 @@ def find_exp_array(N, inverse=False):
     Returns
     -------
     exp_array : array-like
+
     """
     N = int(N)
     exp_array = np.zeros(N, dtype=np.clongdouble)
@@ -173,6 +176,7 @@ def find_exp_array2(N, inverse=False):
     Returns
     -------
     exp_array : array-like
+
     """
     # First compute the usual fft array
     exp_array = find_exp_array(2 * N, inverse=inverse)
@@ -195,6 +199,7 @@ def dft(td_data, exp_array=None, return_double=False, inverse=False):
     Returns
     -------
     fd_data : array
+
     """
     N = len(td_data)
 
@@ -234,13 +239,14 @@ def rdft(td_data, exp_array=None, return_double=False, return_full=False):
     Parameters
     ----------
     td_data : array
-    exp_array : 'array-like'
-    return_double : True or False
-    inverse : True or False
+    exp_array : `float`, array-like, optional
+    return_double : bool, optional
+    return_full : bool, optional
 
     Returns
     -------
     fd_data : array
+
     """
     N = len(td_data)
     N_out = N // 2 + 1
@@ -288,13 +294,15 @@ def irdft(fd_data, exp_array=None, return_double=False, N=None, normalize=True):
     Parameters
     ----------
     fd_data : array
-    exp_array : 'array-like'
-    return_double : True or False
-    inverse : True or False
+    exp_array : `float`, array-like, optional
+    return_double : bool, optional
+    N : `int`, optional
+    normalize : bool, optional
 
     Returns
     -------
-    td_data : float, array-like
+    td_data : `float`, array-like
+
     """
     N_in = len(fd_data)
 
@@ -367,6 +375,7 @@ def fft(td_data, prime_factors=None, exp_array=None, return_double=False,
     Returns
     -------
     fd_data : array-like
+
     """
     N = len(td_data)
 
@@ -469,6 +478,7 @@ def ifft(fd_data, prime_factors=None, exp_array=None, return_double=False, norma
     Returns
     -------
     td_data : array-like
+
     """
     if normalize:
         if return_double:
@@ -484,7 +494,7 @@ def ifft(fd_data, prime_factors=None, exp_array=None, return_double=False, norma
 
 def prime_fft(td_data, return_double=False, inverse=False, exp_array2=None, M=None,
               prime_factors=None, exp_array=None):
-    """
+    r"""
     Bluestein's algorithm for FFTs of prime length, for which the Cooley-Tukey algorithm is
     ineffective.  Make the replacement nk -> -(k - n)^2 / 2 + n^2 / 2 + k^2 / 2.
     Then X_k = sum_(n=0)^(N-1) x_n * exp(-2*pi*i*n*k/N) = exp(-pi*i*k^2/N)
@@ -515,15 +525,18 @@ def prime_fft(td_data, return_double=False, inverse=False, exp_array2=None, M=No
 
     Parameters
     ----------
-    fd_data : array
+    td_data : array
+    return_double : bool, optional
+    inverse : bool, optional
+    exp_array2 : `float`, array-like, optional
+    M : float, optional
     prime_factors : float, int
-    exp_array : 'array-like'
-    return_double : True or False
-    normalize : True or False
+    exp_array : `float`, array-like, optional
 
     Returns
     -------
-    td_data : array-like
+    fd_data : `complex`, array-like
+
     """
     N = len(td_data)
 
@@ -574,6 +587,7 @@ def prime_irfft(fd_data, return_double=False, N=None, normalize=True, exp_array2
     Returns
     -------
     td_data : array-like
+
     """
     N_in = len(fd_data)
     if N is None:
@@ -645,6 +659,7 @@ def prime_rfft(td_data, return_double=False, return_full=False, exp_array2=None,
     Returns
     -------
     fd_data : array-like
+
     """
     N = len(td_data)
     N_out = N // 2 + 1
@@ -707,6 +722,7 @@ def rfft(td_data, prime_factors=None, exp_array=None, return_double=False,
     Returns
     -------
     fd_data : array-like
+
     """
     N = len(td_data)
     N_out = N // 2 + 1
@@ -846,7 +862,6 @@ def irfft(fd_data, prime_factors=None, exp_array=None, return_double=False,
     prime_factors : float, int
     exp_array : 'array-like'
     return_double : True or False
-    return_full : True or False
     M_fft : float
     M_fft_prime_factors : float, int
     M_fft_exp_array2 : 'array-like'
@@ -859,6 +874,7 @@ def irfft(fd_data, prime_factors=None, exp_array=None, return_double=False,
     Returns
     -------
     fd_data : array-like
+
     """
     N_in = len(fd_data)
 
@@ -998,7 +1014,8 @@ def mat_times_vec(mat, vec):
 
     Returns
     -------
-    outvec / outvec[-1] : array-like
+    output : array-like
+
     """
     N = len(mat)
     n = len(vec)
@@ -1009,25 +1026,34 @@ def mat_times_vec(mat, vec):
         reordered_mat[:i] += mat[i:0:-1]
         outvec[i] = np.matmul(vec, reordered_mat)
     # Normalize
-    return outvec / outvec[-1]
+    output = outvec / outvec[-1]
+
+    return output
 
 
 def DPSS(N, alpha, return_double=False, max_time=10):
     """
     DPSS window
+
     Compute a discrete prolate spheroidal sequence (DPSS) window,
     which maximizes the energy concentration in the central lobe.
 
     Parameters
     ----------
-    return_double : True or False
-    max_time : int
-    N : float, int
-    alpha : float, int
+    N : float
+        length
+    alpha : float
+        parameter
+    return_double : bool, optional
+        double or longdouble
+    max_time : int, optional
+        maximum
 
     Returns
     -------
-    np.concatenate((dpss, dpss[::-1][N % 2:])) : array-like
+    output : `longdouble`, array-like
+        the filter
+
     """
     N = int(N)
 
@@ -1091,15 +1117,10 @@ def DPSS(N, alpha, return_double=False, max_time=10):
 
     if return_double:
         dpss = np.float64(dpss)
-    return np.concatenate((dpss, dpss[::-1][N % 2:]))
 
+    output = np.concatenate((dpss, dpss[::-1][N % 2:]))
 
-factorials_inv = np.zeros(1755, dtype=np.longdouble)
-factorials_inv[0] = 1.0
-current = 1
-for n in range(1, 1755):
-    current *= n
-    factorials_inv[n] = 1.0 / np.longdouble(str(current))
+    return output
 
 
 # Dolph-Chebyshev window
@@ -1342,7 +1363,7 @@ def freqresp(filt, delay_samples=0, samples_per_lobe=8, return_double=False):
     # Start with zeros
     filt_prime = np.zeros(N_prime, dtype=np.longdouble)
     # The beginning and end have filter coefficients
-    filt_prime[: N - delay_samples] = np.longdouble(filt[delay_samples:])
+    filt_prime[:N - delay_samples] = np.longdouble(filt[delay_samples:])
     if delay_samples > 0:
         filt_prime[-delay_samples:] = np.longdouble(filt[:delay_samples])
 
