@@ -14,8 +14,20 @@
 # You should have received a copy of the GNU General Public License along with
 # pyDARM. If not, see <https://www.gnu.org/licenses/>.
 
-from . import (sensing, actuation, utils, darm, pcal, measurement, uncertainty,
-               plot, calcs, FIR, FIRtools)
+from . import (
+    sensing,
+    actuation,
+    utils,
+    darm,
+    pcal,
+    measurement,
+    uncertainty,
+    plot,
+    calcs,
+    FIR,
+    FIRtools,
+)
+
 
 try:
     from ._version import version as __version__
