@@ -15,6 +15,7 @@
 # pyDARM. If not, see <https://www.gnu.org/licenses/>.
 
 import numpy as np
+
 from scipy import signal, io
 
 from .utils import digital_delay_filter
@@ -37,6 +38,12 @@ class ActuationModel(Model):
         # TODO need some way of validating the configuration file is
         # appropriate
 
+        # Setting up to the list to save loading in the suspension filters
+        self.tst_digital_filter_response_list = []
+        self.pum_digital_filter_response_list = []
+        self.uim_digital_filter_response_list = []
+        self.filter_response_freq_list = []
+
     def analog_driver_response(self, frequencies):
         """
         The transfer function of the analog driver electronics. By default,
@@ -285,23 +292,41 @@ class ActuationModel(Model):
 
         """
 
-        tst_isc_inf_filter_response, tst_lock_filter_response, \
-            tst_drive_align_filter_response, pum_lock_filter_response, \
-            pum_drive_align_filter_response, uim_lock_filter_response, \
-            uim_drive_align_filter_response = \
-            self.sus_digital_filters(frequencies)
+        use_index = False
+        idx = 0
+        for i, saved_freq_array in enumerate(self.filter_response_freq_list):
+            if np.atleast_1d(np.atleast_1d(frequencies) == np.atleast_1d(saved_freq_array)).all():
+                idx = i
+                use_index = True
 
-        # Start with TST stage filters
-        tst_digital_filter_response = tst_isc_inf_filter_response * \
-            tst_lock_filter_response * tst_drive_align_filter_response
+        if use_index:
+            tst_digital_filter_response = self.tst_digital_filter_response_list[idx]
+            pum_digital_filter_response = self.pum_digital_filter_response_list[idx]
+            uim_digital_filter_response = self.uim_digital_filter_response_list[idx]
 
-        pum_digital_filter_response = tst_isc_inf_filter_response * \
-            tst_lock_filter_response * pum_lock_filter_response * \
-            pum_drive_align_filter_response
+        else:
+            tst_isc_inf_filter_response, tst_lock_filter_response, \
+                tst_drive_align_filter_response, pum_lock_filter_response, \
+                pum_drive_align_filter_response, uim_lock_filter_response, \
+                uim_drive_align_filter_response = \
+                self.sus_digital_filters(frequencies)
 
-        uim_digital_filter_response = tst_isc_inf_filter_response * \
-            tst_lock_filter_response * pum_lock_filter_response * \
-            uim_lock_filter_response * uim_drive_align_filter_response
+            # Start with TST stage filters
+            tst_digital_filter_response = tst_isc_inf_filter_response * \
+                tst_lock_filter_response * tst_drive_align_filter_response
+
+            pum_digital_filter_response = tst_isc_inf_filter_response * \
+                tst_lock_filter_response * pum_lock_filter_response * \
+                pum_drive_align_filter_response
+
+            uim_digital_filter_response = tst_isc_inf_filter_response * \
+                tst_lock_filter_response * pum_lock_filter_response * \
+                uim_lock_filter_response * uim_drive_align_filter_response
+
+            self.filter_response_freq_list.append(frequencies)
+            self.tst_digital_filter_response_list.append(tst_digital_filter_response)
+            self.pum_digital_filter_response_list.append(pum_digital_filter_response)
+            self.uim_digital_filter_response_list.append(uim_digital_filter_response)
 
         return uim_digital_filter_response, \
             pum_digital_filter_response, \
@@ -769,7 +794,7 @@ class ActuationModel(Model):
         [uim_digital_filter_response,
          pum_digital_filter_response,
          tst_digital_filter_response] = self.sus_digital_filters_response(
-             frequencies)
+            frequencies)
 
         # Choose the right stage
         if stage in ['UIM', 'L1']:
