@@ -16,6 +16,7 @@
 
 import numpy as np
 from scipy import signal
+
 from .utils import load_foton_export_tf, digital_delay_filter
 from .digital import daqdownsamplingfilters
 from .model import Model
@@ -78,6 +79,10 @@ class CALCSModel(DARMModel):
         foton_freq, foton_tf = load_foton_export_tf(
             self.dpath(self.calcs.foton_invsensing_tf))
 
+        # apply gain, default is 1.0
+        invsensing_gain = getattr(self, 'invsensing_gain', 1.0)
+        foton_tf *= invsensing_gain
+
         # Take ratio of true optical response / foton response where
         # "foton response" is already the approximated
         # LP_foton / (opt resp)_foton, so we just need to multiply the two
@@ -203,7 +208,9 @@ class CALCSModel(DARMModel):
 
 
         The gain (N/ct) of the CALCS model is assumed to be exactly the same
-        as what is in the reference model, so that is used.
+        as what is in the reference model, so that is used. If any digital
+        gain is present as <arm>_<stage>_analog_gain, then it will be
+        applied here.
 
         If there is anything in the "coiloutf" key-value pair, then this
         transfer function is also used, but this would be used only in
@@ -261,8 +268,6 @@ class CALCSModel(DARMModel):
         # put in the gain and the DARM feedback sign
         # FOTON export is in m/N, here we apply the N/ct from the model
         # assuming that this has been replicated perfectly in CALCS.
-        # Note the DARM feedback sign is applied here as well, from the
-        # model.
         if (getattr(self.actuation, f'{arm.lower()}arm') is not None and
                 stage.lower() in ['uim', 'pum', 'tst']):
             if ((stage.lower() in ['uim', 'pum'] and
@@ -276,7 +281,6 @@ class CALCSModel(DARMModel):
                         getattr(getattr(self.actuation, f'{arm.lower()}arm'),
                                 f'{stage.lower()}_npv2') != '')):
                 act_arm_model = getattr(self.actuation, f'{arm.lower()}arm')
-                # tf_ana_interp *= act_arm_model.darm_feedback_sign
                 if stage.lower() == 'uim':
                     tf_ana_interp *= act_arm_model.uim_dc_gain_Npct()
                 elif stage.lower() == 'pum':
@@ -284,6 +288,10 @@ class CALCSModel(DARMModel):
                 else:
                     tf_ana_interp *= act_arm_model.tst_dc_gain_Npct()
 
+        # apply CALCS digital gain, default is 1
+        act_gain = getattr(self, f'{arm}arm_{stage.lower()}_analog_gain', 1.0)
+        tf_ana_interp *= act_gain
+
         out_tf = (out_mtrx *
                   tf_dig_interp *
                   tf_ana_interp)
@@ -837,3 +845,174 @@ class CALCSModel(DARMModel):
         tf = deltal_ext / pcal_corr
 
         return tf
+
+    def compute_epics_records(self, endstation=False):
+        """Generate EPICS values for CALCS front-end model
+
+        This is a LIGO specific function for generating front end
+        calibration EPICS records.
+
+        See T1700106-v10 and G1601472 for additional details
+
+        Parameters
+        ----------
+        endstation : `bool`, optional
+            When false (default), the correction is computed for CAL-CS
+            PCAL channel, which includes 1 16k clock cycle delay that we must
+            compensate (undo). Otherwise, when true, the correction is computed
+            at the end station, which does not include 1 16k clock cycle delay.
+
+        Returns
+        -------
+        out : dict
+            Dictionary of EPICS records
+
+        """
+        f_uim = self.calcs.cal_line_sus_uim_frequency
+        f_pum = self.calcs.cal_line_sus_pum_frequency
+        f_tst = self.calcs.cal_line_sus_tst_frequency
+        f_pcal1 = self.calcs.cal_line_sus_pcal_frequency
+        f_pcal2 = self.calcs.cal_line_sens_pcal_frequency
+        f_pcal3 = self.calcs.cal_line_high_pcal_frequency
+        f_pcalx_cmp = self.calcs.cal_line_cmp_pcalx_frequency
+        f_pcaly_cmp = self.calcs.cal_line_cmp_pcaly_frequency
+
+        freq = np.array(
+            [f_pcal1, f_uim, f_pum, f_tst, f_pcal2, f_pcal3, f_pcalx_cmp,
+             f_pcaly_cmp])
+
+        R = self.compute_response_function(freq)
+
+        daqdownsampling = daqdownsamplingfilters(2**14, 2**9, 'biquad', 'v3')
+        daqdownsampling_response = signal.dfreqresp(daqdownsampling,
+                                                    2.0*np.pi*freq/2**14)[1]
+
+        # Pcal corrections
+        # See T1700106
+        pcal_correction = self.pcal.compute_pcal_correction(freq, endstation)
+        PCAL_LINE1_CORRECTION = (
+            pcal_correction[0] * np.exp(-2*np.pi*1j*freq[0]/16384))
+        PCAL_LINE2_CORRECTION = (
+            pcal_correction[4] * np.exp(-2*np.pi*1j*freq[4]/16384))
+        PCAL_LINE3_CORRECTION = (
+            pcal_correction[5] * np.exp(-2*np.pi*1j*freq[5]/16384))
+
+        # We divide out the pcal reference arm sign since we know which arm
+        # the next two values will be assigned to. This makes the next two
+        # values constant no matter what arm the PCal calibration lines are
+        # assigned to
+        PCAL_X_COMPARE_CORRECTION = (
+            pcal_correction[6] / self.pcal.ref_pcal_2_darm_act_sign *
+            np.exp(-2*np.pi*1j*freq[6]/16384))
+        PCAL_Y_COMPARE_CORRECTION = (
+            -1 * pcal_correction[7] / self.pcal.ref_pcal_2_darm_act_sign *
+            np.exp(-2*np.pi*1j*freq[7]/16384))
+
+        # this does not include any OMC to SUS model jump delay
+        uim_actuation_epics = self.actuation.stage_super_actuator_drivealign(
+            freq, stage='UIM')
+        pum_actuation_epics = self.actuation.stage_super_actuator_drivealign(
+            freq, stage='PUM')
+        tst_actuation_epics = self.actuation.stage_super_actuator_drivealign(
+            freq, stage='TST')
+
+        # super actuator stage
+        # this does include OMC to SUS model jump delay
+        uim = self.actuation.stage_super_actuator(freq, stage='UIM')
+        pum = self.actuation.stage_super_actuator(freq, stage='PUM')
+        tst = self.actuation.stage_super_actuator(freq, stage='TST')
+
+        # Actuator EPICS
+        # Need to divide out a DAQ downsampling filter from the reference A
+        # values because channels sampled at 512 Hz have an AA filter applied
+        # See T1700106
+        SUS_LINE1_REF_INVA_UIM_RESPRATIO = (
+            (1.0/(uim_actuation_epics[1]/daqdownsampling_response[1])) *
+            (1.0/R[0]) * R[1] * np.exp(2*np.pi*1j*freq[1]/16384))
+        SUS_LINE2_REF_INVA_PUM_RESPRATIO = (
+            (1.0/(pum_actuation_epics[2]/daqdownsampling_response[2])) *
+            (1.0/R[0]) * R[2] * np.exp(2*np.pi*1j*freq[2]/16384))
+        SUS_LINE3_REF_INVA_TST_RESPRATIO = (
+            (1.0/(tst_actuation_epics[3]/daqdownsampling_response[3])) *
+            (1.0/R[0]) * R[3] * np.exp(2*np.pi*1j*freq[3]/16384))
+
+        # Need to add a phase delay to the SUS excitation because the
+        # SUS channel acquires a phase delay from the model jump
+        # to CAL-CS and the calculation expects no delay (as in GDS).
+        # Note that if a replica SUS excitation is used in the CAL-CS model
+        # then these should actually be set to zero.
+        SUS_LINE1_SUS_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[1]/16384), deg=True))
+        SUS_LINE2_SUS_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[2]/16384), deg=True))
+        SUS_LINE3_SUS_DEMOD_PHASE = (
+            np.angle(np.exp(-2*np.pi*1j*freq[3]/16384), deg=True))
+
+        # Compute the sensing function without the optical response but need to
+        # include the optical gain
+        coupled_cavity = self.sensing.optical_response(
+            self.sensing.coupled_cavity_pole_frequency,
+            self.sensing.detuned_spring_frequency,
+            self.sensing.detuned_spring_q,
+            pro_spring=self.sensing.is_pro_spring)
+        sensing_no_cavity_response = (
+            self.sensing.compute_sensing(freq) /
+            signal.freqresp(coupled_cavity, 2.0*np.pi*freq)[1])
+        PCAL_LINE2_REF_C_NOCAVPOLE = sensing_no_cavity_response[4]
+        PCAL_LINE1_REF_C_NOCAVPOLE = sensing_no_cavity_response[0]
+
+        # DARM digital filtering
+        darm_digital_filter_response = self.digital.compute_response(freq)
+        PCAL_LINE2_REF_D = darm_digital_filter_response[4]
+        PCAL_LINE1_REF_D = darm_digital_filter_response[0]
+
+        # Actuation
+        PCAL_LINE2_REF_A_TST = tst[4]
+        PCAL_LINE2_REF_A_PUM = pum[4]
+        PCAL_LINE2_REF_A_UIM = uim[4]
+        PCAL_LINE1_REF_A_TST = tst[0]
+        PCAL_LINE1_REF_A_PUM = pum[0]
+        PCAL_LINE1_REF_A_UIM = uim[0]
+
+        # Output - see T1700106
+        return {
+            'CAL-CS_TDEP_PCAL_LINE1_CORRECTION_REAL': PCAL_LINE1_CORRECTION.real,
+            'CAL-CS_TDEP_PCAL_LINE1_CORRECTION_IMAG': PCAL_LINE1_CORRECTION.imag,
+            'CAL-CS_TDEP_SUS_LINE3_REF_INVA_TST_RESPRATIO_REAL': SUS_LINE3_REF_INVA_TST_RESPRATIO.real,  # noqa E501
+            'CAL-CS_TDEP_SUS_LINE3_REF_INVA_TST_RESPRATIO_IMAG': SUS_LINE3_REF_INVA_TST_RESPRATIO.imag,  # noqa E501
+            'CAL-CS_TDEP_SUS_LINE2_REF_INVA_PUM_RESPRATIO_REAL': SUS_LINE2_REF_INVA_PUM_RESPRATIO.real,  # noqa E501
+            'CAL-CS_TDEP_SUS_LINE2_REF_INVA_PUM_RESPRATIO_IMAG': SUS_LINE2_REF_INVA_PUM_RESPRATIO.imag,  # noqa E501
+            'CAL-CS_TDEP_SUS_LINE1_REF_INVA_UIM_RESPRATIO_REAL': SUS_LINE1_REF_INVA_UIM_RESPRATIO.real,  # noqa E501
+            'CAL-CS_TDEP_SUS_LINE1_REF_INVA_UIM_RESPRATIO_IMAG': SUS_LINE1_REF_INVA_UIM_RESPRATIO.imag,  # noqa E501
+            'CAL-CS_TDEP_PCAL_LINE2_REF_C_NOCAVPOLE_REAL': PCAL_LINE2_REF_C_NOCAVPOLE.real,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_C_NOCAVPOLE_IMAG': PCAL_LINE2_REF_C_NOCAVPOLE.imag,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_D_REAL': PCAL_LINE2_REF_D.real,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_D_IMAG': PCAL_LINE2_REF_D.imag,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_TST_REAL': PCAL_LINE2_REF_A_TST.real,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_TST_IMAG': PCAL_LINE2_REF_A_TST.imag,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_PUM_REAL': PCAL_LINE2_REF_A_PUM.real,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_PUM_IMAG': PCAL_LINE2_REF_A_PUM.imag,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_UIM_REAL': PCAL_LINE2_REF_A_UIM.real,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_UIM_IMAG': PCAL_LINE2_REF_A_UIM.imag,
+            'CAL-CS_TDEP_PCAL_LINE2_CORRECTION_REAL': PCAL_LINE2_CORRECTION.real,
+            'CAL-CS_TDEP_PCAL_LINE2_CORRECTION_IMAG': PCAL_LINE2_CORRECTION.imag,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_C_NOCAVPOLE_REAL': PCAL_LINE1_REF_C_NOCAVPOLE.real,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_C_NOCAVPOLE_IMAG': PCAL_LINE1_REF_C_NOCAVPOLE.imag,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_D_REAL': PCAL_LINE1_REF_D.real,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_D_IMAG': PCAL_LINE1_REF_D.imag,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_TST_REAL': PCAL_LINE1_REF_A_TST.real,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_TST_IMAG': PCAL_LINE1_REF_A_TST.imag,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_PUM_REAL': PCAL_LINE1_REF_A_PUM.real,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_PUM_IMAG': PCAL_LINE1_REF_A_PUM.imag,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_UIM_REAL': PCAL_LINE1_REF_A_UIM.real,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_UIM_IMAG': PCAL_LINE1_REF_A_UIM.imag,
+            'CAL-CS_TDEP_PCAL_LINE3_CORRECTION_REAL': PCAL_LINE3_CORRECTION.real,
+            'CAL-CS_TDEP_PCAL_LINE3_CORRECTION_IMAG': PCAL_LINE3_CORRECTION.imag,
+            'CAL-CS_TDEP_PCAL_X_COMPARE_CORRECTION_REAL': PCAL_X_COMPARE_CORRECTION.real,
+            'CAL-CS_TDEP_PCAL_X_COMPARE_CORRECTION_IMAG': PCAL_X_COMPARE_CORRECTION.imag,
+            'CAL-CS_TDEP_PCAL_Y_COMPARE_CORRECTION_REAL': PCAL_Y_COMPARE_CORRECTION.real,
+            'CAL-CS_TDEP_PCAL_Y_COMPARE_CORRECTION_IMAG': PCAL_Y_COMPARE_CORRECTION.imag,
+            'CAL-CS_TDEP_SUS_LINE1_SUS_DEMOD_PHASE': SUS_LINE1_SUS_DEMOD_PHASE,
+            'CAL-CS_TDEP_SUS_LINE2_SUS_DEMOD_PHASE': SUS_LINE2_SUS_DEMOD_PHASE,
+            'CAL-CS_TDEP_SUS_LINE3_SUS_DEMOD_PHASE': SUS_LINE3_SUS_DEMOD_PHASE,
+        }
