@@ -15,25 +15,24 @@
 # pyDARM. If not, see <https://www.gnu.org/licenses/>.
 
 import os
-import numpy as np
 import math
-import emcee
 import json
-import h5py
-from datetime import datetime
-from scipy.signal import freqresp, ZerosPolesGain
 import multiprocessing as mp
-from scipy import signal
+from datetime import datetime
+
+import h5py
+import emcee
+import numpy as np
+from scipy.signal import freqresp, ZerosPolesGain
+from sklearn.gaussian_process import GaussianProcessRegressor
+from sklearn.gaussian_process.kernels import RBF, ConstantKernel
 
 import dttxml
 
 from .sensing import SensingModel
 from .actuation import ActuationModel
 from .pcal import PcalModel
-from .utils import save_gpr_to_hdf5
-
-from sklearn.gaussian_process import GaussianProcessRegressor
-from sklearn.gaussian_process.kernels import RBF, ConstantKernel
+from .utils import save_chain_to_hdf5, save_gpr_to_hdf5
 
 
 class Measurement(object):
@@ -71,9 +70,10 @@ class Measurement(object):
         self.filename = meas_file
 
         # Data accessing object. It is called later.
-        if meas_file.endswith('.xml'):
+        base, ext = os.path.splitext(meas_file)
+        if ext == '.xml':
             self.data_access = dttxml.DiagAccess(meas_file)
-        elif meas_file.endswith('.hdf5'):
+        elif ext == '.hdf5':
             self.data_access = h5py.File(meas_file, 'r')
         else:
             raise TypeError('Invalid file type input.')
@@ -460,7 +460,7 @@ class ProcessMeasurement(object):
         # Return the array of MCMC samples
         return sampler.get_chain(flat=True)
 
-    def save_results_to_json(self, filename, fmin, fmax, mcmc_map_vals, measurement):
+    def save_results_to_json(self, filename, fmin, fmax, mcmc_map_vals, measurement, append=False):
         """
         Save the model, input DTT filenames, coherence threshold values,
         minimum and maximum frequencies used in the MCMC, and the resulting
@@ -478,6 +478,8 @@ class ProcessMeasurement(object):
             array of maximum a postiori values found from the MCMC
         measurement : str
             measurement type; one of 'sensing', 'actuation_x_arm', or 'actuation_y_arm'
+        append : bool
+            if True, append results to existing JSON file, otherwise overwrite
 
         """
         # Prepare the new dict for the JSON archive
@@ -492,11 +494,11 @@ class ProcessMeasurement(object):
 
         measurement_info = {'Measurement analysis {}'.format(
             now.strftime('%d/%m/%Y %H:%M:%S')): {
-            'model_config_file': self.config_file,
+            'model_config_file': str(self.config_file),
             'meas_type': measurement,
             'model_config': config_dict,
-            'loop_meas': self.measurement_1.filename,
-            'pcal_meas': self.measurement_2.filename,
+            'loop_meas': str(self.measurement_1.filename),
+            'pcal_meas': str(self.measurement_2.filename),
             'loop_tup': self.meas_tup_1,
             'pcal_tup': self.meas_tup_2,
             'loop_cohthresh': self.meas1_cohThresh,
@@ -507,7 +509,7 @@ class ProcessMeasurement(object):
         # Either read in and make an addition to the JSON archive or create
         # the file if it didn't exist
         filename = os.path.normpath(filename)
-        if os.path.exists(filename):
+        if os.path.exists(filename) and append is True:
             # JSON doesn't like to read/write files in append mode for some
             # reason
             with open(filename, 'r') as f:
@@ -654,7 +656,14 @@ class ProcessMeasurement(object):
 
         return output
 
-    def _gpr(self, freq, residuals, RBF_length_scale, RBF_length_scale_limits):
+    def stack_measurements(self):
+        """ This method is meant to be overloaded by inherited classes """
+
+        raise Exception('This method is intended to be overloaded by child'
+                        'classes')
+
+    def _gpr(self, freq, meas_list, fmin, fmax, fmin_list, fmax_list,
+             RBF_length_scale, RBF_length_scale_limits, gpr_flim):
         """
         Run a GPR over the processed measurement data
 
@@ -677,9 +686,41 @@ class ProcessMeasurement(object):
             Standard deviation for the GPR
         cov : `float`, array-like
             Covariance matrix for the GPR
+        stacked_meas : `list`, array-like
+            list of stacked measurement frequencies, transfer functions,
+            rescaled transfer functions (to reference), uncertainties, and
+            residuals
+        tdcfs : list
+            list of time dependent correction factors for each measurement
 
         """
 
+        stacked_meas, tdcfs = self.stack_measurements(meas_list, fmin, fmax,
+                                                      fmin_list, fmax_list)
+
+        for idx, this_meas in enumerate(stacked_meas):
+            # Create a mask array where True means the data values should be
+            # ignored while False indicates the data should be included
+            if gpr_flim is not None:
+                mask = np.ma.masked_outside(
+                    this_meas[0], gpr_flim[0], gpr_flim[1]).mask
+            else:
+                mask = np.array([False] * len(this_meas[0]))
+
+            # Create / append to an array of frequencies, residuals, and
+            # uncertainties
+            if idx == 0:
+                residuals = np.array(
+                    [this_meas[0][~mask],
+                     this_meas[4][~mask],
+                     this_meas[3][~mask]]).T
+            else:
+                residuals = np.vstack(
+                    (residuals, np.array(
+                        [this_meas[0][~mask],
+                         this_meas[4][~mask],
+                         this_meas[3][~mask]]).T))
+
         # Now clean up, extract, and get in the right shape for the GPR
         fdata = np.log10(np.real(residuals[None, :, 0].T))
         meas = np.hstack((np.real(residuals[None, :, 1].T), np.imag(residuals[None, :, 1].T)))
@@ -708,7 +749,7 @@ class ProcessMeasurement(object):
         sigma = sigma[:, 0]
         cov = cov[:, :, 0]
 
-        return y_pred, sigma, cov
+        return y_pred, sigma, cov, stacked_meas, tdcfs
 
 
 class ProcessSensingMeasurement(ProcessMeasurement):
@@ -739,7 +780,7 @@ class ProcessSensingMeasurement(ProcessMeasurement):
         Coherence threshold that values in measurement 2 must be above to be
         included; valid values in the range [0,1]. (default = 0)
     json_results_file : `str`, optional
-        Filename of a JSON file
+        Filename of a JSON file when stacking measurements or running GPR
 
     """
 
@@ -838,12 +879,12 @@ class ProcessSensingMeasurement(ProcessMeasurement):
                                     2.0*np.pi*frequencies)[1]
             response *= ref_response / this_response
 
-        response *= 1.0/kappa_c
+        response *= 1.0 / kappa_c
 
         return response
 
     def run_mcmc(self, fmin=None, fmax=None, priors_bound=None,
-                 save_to_file=None, **kwargs):
+                 save_map_to_file=None, save_chain_to_file=None, **kwargs):
         """
         This method is what is used to process the sensing function measurement
         data
@@ -865,8 +906,11 @@ class ProcessSensingMeasurement(ProcessMeasurement):
             should be in form of a 2D array, 5X2. First (second) element of
             each row corresponds to the lower (upper) bound of each
             parameters: gain, pole freq, spring freq, inv Q, residual delay.
-        save_to_file : `str`, optional
-            Filename (JSON) to save the data from this result
+        save_map_to_file : `str`, optional
+            Filename (JSON) to save the maximum a postiori values from this
+            result
+        save_chain_to_file : `str`, optional
+            Filename (HDF5) to save the postirior chain from this result
         kwargs : optional
             other kwargs are passed to ProcessMeasurement._mcmc
 
@@ -881,10 +925,18 @@ class ProcessSensingMeasurement(ProcessMeasurement):
             - 4 is the residual time delay in units of seconds
 
         """
-        global meas_type_gl, is_pro_spring_gl
+        global meas_type_gl, is_pro_spring_gl, include_spring_gl
         meas_type_gl = self.meas_type
         is_pro_spring_gl = self.sensing.is_pro_spring
 
+        if (hasattr(self.sensing, 'include_spring') and
+                getattr(self.sensing, 'include_spring') != ''):
+            # Just in case, be extra sure that we pass a boolean
+            include_spring_gl = bool(self.sensing.include_spring)
+        else:
+            # default to True if nothing given
+            include_spring_gl = True
+
         (frequencies, processed_measurement_response,
          processed_measurement_response_unc) = \
             self.get_processed_measurement_response()
@@ -913,12 +965,22 @@ class ProcessSensingMeasurement(ProcessMeasurement):
 
         chain = self._mcmc(xdata, ydata, yerr, priors, priors_bound, **kwargs)
 
+        if not include_spring_gl:
+            chain[:, 2] = np.zeros(len(chain[:, 0]))
+            chain[:, 3] = 1e-3 * np.ones(len(chain[:, 0]))
+
         # Fix 1/Q ==> Q
         chain[:, 3] = 1/chain[:, 3]
 
-        if save_to_file is not None:
+        if save_map_to_file is not None:
             self.save_results_to_json(
-                save_to_file, fmin, fmax, np.median(chain, axis=0), self.meas_type)
+                save_map_to_file, fmin, fmax, np.median(chain, axis=0),
+                self.meas_type)
+
+        if save_chain_to_file is not None:
+            save_chain_to_hdf5(
+                save_chain_to_file, self.sensing.config_to_dict(), fmin, fmax,
+                self.meas_type, chain)
 
         return chain
 
@@ -1046,8 +1108,12 @@ class ProcessSensingMeasurement(ProcessMeasurement):
 
         Returns
         -------
-        output : list
-            list of normalized measurements
+        output : `list`, array-like
+            list of stacked measurement frequencies, transfer functions,
+            rescaled transfer functions (to reference), uncertainties, and
+            residuals
+        tdcfs : list
+            list of time dependent correction factors for each measurement
 
         """
         assert ((len(measurement_list) == len(fmin_list) and
@@ -1058,9 +1124,10 @@ class ProcessSensingMeasurement(ProcessMeasurement):
                  len(fmin_list) == 0))
 
         output = []
+        tdcfs = []
 
-        ref_date, ref_map, ref_spring = self.query_results_from_json(self.meas_type, fmin,
-                                                                     fmax)
+        ref_date, ref_map, ref_spring = self.query_results_from_json(
+            self.meas_type, fmin, fmax)
 
         for idx, meas in enumerate(measurement_list):
             this_freq, this_resp, this_resp_unc = \
@@ -1088,24 +1155,41 @@ class ProcessSensingMeasurement(ProcessMeasurement):
             # because we'll let the GPR take care of covering any variations
             # due to a potentially poorly modeled optical spring
             this_response_zpk = SensingModel.optical_response(
-                this_map[1], ref_map[2], ref_map[3], this_spring)
+                this_map[1], ref_map[2], ref_map[3], ref_spring)
             ref_response_zpk = SensingModel.optical_response(
-                ref_map[1], ref_map[2], ref_map[3], this_spring)
+                ref_map[1], ref_map[2], ref_map[3], ref_spring)
 
+            # Compute the transfer function response for the current
+            # measurement model and the reference model
             this_model = this_map[0] * freqresp(
                 this_response_zpk, 2.0*np.pi*this_freq)[1]
             ref_model = ref_map[0] * freqresp(
                 ref_response_zpk, 2.0*np.pi*this_freq)[1]
 
-            this_resp *= ref_model / this_model
+            # The residual for this measurement is just the measured response
+            # divided by the model for this measurement
+            this_resid = this_resp / this_model
 
-            output.append([this_freq, this_resp, this_resp_unc])
+            # Now rescale this measurement by multiplying this measurement
+            # residuals by the reference model. This is equivalent to dividing
+            # out the model for this measurement and multiplying in the
+            # reference model
+            this_resp_r = ref_model * this_resid
 
-        return output
+            # Compute kappa_c as the ratio of the current measured gain to the
+            # reference gain. We don't actually use this here other than as an
+            # output
+            kappa_c = this_map[0] / ref_map[0]
+
+            output.append([this_freq, this_resp, this_resp_r, this_resp_unc, this_resid])
+            tdcfs.append([kappa_c, this_map[1], ref_map[2], ref_map[3]])
+
+        return output, tdcfs
 
     def run_gpr(self, frequencies, measurement_list, fmin=None, fmax=None,
                 fmin_list=None, fmax_list=None,
-                RBF_length_scale=1.0, RBF_length_scale_limits=(0.5, 1.5), save_to_file=None):
+                RBF_length_scale=1.0, RBF_length_scale_limits=(0.5, 1.5),
+                gpr_flim=None, save_to_file=None):
         """
         Run a Gaussian Process Regression on a set of sensing function measurements
 
@@ -1127,6 +1211,8 @@ class ProcessSensingMeasurement(ProcessMeasurement):
             Initial length scale guess for the RBF term of the GPR fit (in logarithmic frequency)
         RBF_length_scale_limits : pair of floats >= 0, optional
             The lower and upper bound on RBF_length_scale
+        gpr_flim : `tuple`, optional
+            Minimum and maximum frequency points to include in the GPR, (fmin, fmax)
         save_to_file : str, optional
             Filename (HDF5) to save the data from this result
 
@@ -1138,8 +1224,12 @@ class ProcessSensingMeasurement(ProcessMeasurement):
             Standard deviation for the GPR
         cov : `float`, array-like
             Covariance matrix for the GPR
-        residuals : `complex128`, array-like
-            residuals of the list of measurements
+        stacked_meas : `list`, array-like
+            list of stacked measurement frequencies, transfer functions,
+            rescaled transfer functions (to reference), uncertainties, and
+            residuals
+        tdcfs : list
+            list of time dependent correction factors for each measurement
 
         """
         assert ((len(measurement_list) == len(fmin_list) and
@@ -1149,39 +1239,17 @@ class ProcessSensingMeasurement(ProcessMeasurement):
                 (len(measurement_list) == len(fmax_list) and
                  len(fmin_list) == 0))
 
-        ref_date, MAP, ref_spring = self.query_results_from_json(self.meas_type, fmin,
-                                                                 fmax)
-
-        stacked_meas = self.stack_measurements(measurement_list, fmin, fmax,
-                                               fmin_list, fmax_list)
-
-        for idx, _ in enumerate(stacked_meas):
-            opticalResponse_ref_tf = MAP[0] * \
-               signal.freqresp(SensingModel.optical_response(MAP[1], MAP[2],
-                                                             MAP[3]),
-                               2.0*np.pi*stacked_meas[idx][0])[1]
-            if idx == 0:
-                residuals = np.array(
-                    [stacked_meas[idx][0],
-                     stacked_meas[idx][1]/opticalResponse_ref_tf,
-                     stacked_meas[idx][2]]).T
-            else:
-                residuals = np.vstack(
-                    (residuals, np.array(
-                        [stacked_meas[idx][0],
-                         stacked_meas[idx][1]/opticalResponse_ref_tf,
-                         stacked_meas[idx][2]]).T))
-
-        y_pred, sigma, cov = self._gpr(frequencies, residuals,
-                                       RBF_length_scale,
-                                       RBF_length_scale_limits)
+        y_pred, sigma, cov, stacked_meas, tdcfs = self._gpr(
+            frequencies, measurement_list, fmin, fmax,
+            fmin_list, fmax_list, RBF_length_scale,
+            RBF_length_scale_limits, gpr_flim)
 
         # save the result to HDF5
         if save_to_file is not None:
             save_gpr_to_hdf5(save_to_file, self.sensing, 'sensing', y_pred,
                              cov, frequencies)
 
-        return y_pred, sigma, cov, residuals
+        return y_pred, sigma, cov, stacked_meas, tdcfs
 
 
 class ProcessActuationMeasurement(ProcessMeasurement):
@@ -1215,7 +1283,7 @@ class ProcessActuationMeasurement(ProcessMeasurement):
         Coherence threshold that values in measurement 2 must be above to be
         included; valid values in the range [0,1]. (default = 0)
     json_results_file : `str`, optional
-        Filename of a JSON file
+        Filename of a JSON file when stacking measurements or running GPR
 
     """
 
@@ -1299,7 +1367,7 @@ class ProcessActuationMeasurement(ProcessMeasurement):
         return response / kappa_Ai
 
     def run_mcmc(self, fmin=None, fmax=None, priors_bound=None,
-                 save_to_file=None, **kwargs):
+                 save_map_to_file=None, save_chain_to_file=None, **kwargs):
         """
         This method is what is used to process the actuation function
         measurement data
@@ -1319,8 +1387,11 @@ class ProcessActuationMeasurement(ProcessMeasurement):
             should be in form of a 2D array, 2X2. First (second) element of
             each row corresponds to the lower (upper) bound of each
             parameters: gain, residual delay.
-        save_to_file : `str`, optional
-            Filename (JSON) to save the data from this result
+        save_map_to_file : `str`, optional
+            Filename (JSON) to save the maximum a postiori values from this
+            result
+        save_chain_to_file : `str`, optional
+            Filename (HDF5) to save the postirior chain from this result
         kwargs : optional
             other kwargs are passed to ProcessMeasurement._mcmc
 
@@ -1362,14 +1433,20 @@ class ProcessActuationMeasurement(ProcessMeasurement):
 
         chain = self._mcmc(xdata, ydata, yerr, priors, priors_bound, **kwargs)
 
-        if save_to_file is not None:
+        if save_map_to_file is not None:
             self.save_results_to_json(
-                save_to_file, fmin, fmax, np.median(chain, axis=0), self.meas_type)
+                save_map_to_file, fmin, fmax, np.median(chain, axis=0),
+                self.meas_type)
+
+        if save_chain_to_file is not None:
+            save_chain_to_hdf5(
+                save_chain_to_file, self.actuation.config_to_dict(), fmin, fmax,
+                self.meas_type, chain)
 
         return chain
 
     def stack_measurements(self, measurement_list, fmin=None, fmax=None,
-                           fmin_list=None, fmax_list=None):
+                           fmin_list=None, fmax_list=None, use_ini_reference=True):
         """
         Stack measurements together where each measurement's model for known
         filtering and digital filters is removed and the time dependence is
@@ -1394,11 +1471,18 @@ class ProcessActuationMeasurement(ProcessMeasurement):
             list of minimum frequencies used for the MCMC or None
         fmax_list : `float` list, optional
             list of maximum frequencies used for the MCMC or None
+        use_ini_reference : `boolean'
+            flag to use the parameters for the reference model
+            from the ini (default = True)
 
         Returns
         -------
-        output : list
-            list of normalized measurements
+        output : `list`, array-like
+            list of stacked measurement frequencies, transfer functions,
+            rescaled transfer functions (to reference), uncertainties, and
+            residuals
+        tdcfs : list
+            list of time dependent correction factors for each measurement
 
         """
         assert ((len(measurement_list) == len(fmin_list) and
@@ -1409,9 +1493,9 @@ class ProcessActuationMeasurement(ProcessMeasurement):
                  len(fmin_list) == 0))
 
         output = []
+        tdcfs = []
 
-        ref_map = self.query_results_from_json(
-            self.meas_type, fmin, fmax)[1]
+        ref_map = self.query_results_from_json(self.meas_type, fmin, fmax)[1]
 
         for idx, meas in enumerate(measurement_list):
             this_freq, this_resp, this_resp_unc = \
@@ -1428,17 +1512,40 @@ class ProcessActuationMeasurement(ProcessMeasurement):
                 this_map = meas.query_results_from_json(
                     self.meas_type, fmin_list[idx], fmax_list[idx])[1]
 
-            # Note that we only scale by the gain and not any residual time
+            # We want to compute the transfer function as though we were
+            # measuring the detector at the same reference time, so we need
+            # to divide out the actuation response from this measurement **
+            # and then multiply in the actuation response from the reference
+            # measurement.
+            # ** Note that we only scale by the gain and not any residual time
             # delay
-            this_resp *= ref_map[0] / this_map[0]
+            this_model = this_map[0]
+            ref_model = ref_map[0]
 
-            output.append([this_freq, this_resp, this_resp_unc])
+            # The residual for this measurement is just the measured response
+            # divided by the model for this measurement
+            this_resid = this_resp / this_model
 
-        return output
+            # Now rescale this measurement by multiplying this measurement
+            # residuals by the reference model. This is equivalent to dividing
+            # out the model for this measurement and multiplying in the
+            # reference model
+            this_resp_r = ref_model * this_resid
+
+            # Compute kappa_a as the ratio of the current measured gain to the
+            # reference gain. We don't actually use this here other than as an
+            # output
+            kappa_a = this_map[0] / ref_map[0]
+
+            output.append([this_freq, this_resp, this_resp_r, this_resp_unc, this_resid])
+            tdcfs.append(kappa_a)
+
+        return output, tdcfs
 
     def run_gpr(self, frequencies, measurement_list, fmin=None, fmax=None,
                 fmin_list=None, fmax_list=None,
-                RBF_length_scale=1.0, RBF_length_scale_limits=(0.5, 1.5), save_to_file=None):
+                RBF_length_scale=1.0, RBF_length_scale_limits=(0.5, 1.5),
+                gpr_flim=None, save_to_file=None):
         """
         Run a Gaussian Process Regression on a set of actuation function measurements
 
@@ -1460,6 +1567,8 @@ class ProcessActuationMeasurement(ProcessMeasurement):
             Initial length scale guess for the RBF term of the GPR fit (in logarithmic frequency)
         RBF_length_scale_limits : pair of floats >= 0, optional
             The lower and upper bound on RBF_length_scale
+        gpr_flim : `tuple`, optional
+            Minimum and maximum frequency points to include in the GPR, (fmin, fmax)
         save_to_file : str, optional
             Filename (HDF5) to save the data from this result
 
@@ -1471,8 +1580,12 @@ class ProcessActuationMeasurement(ProcessMeasurement):
             Standard deviation for the GPR
         cov : `float`, array-like
             Covariance matrix for the GPR
-        residuals : `complex128`, array-like
-            residuals of the list of measurements
+        stacked_meas : `list`, array-like
+            list of stacked measurement frequencies, transfer functions,
+            rescaled transfer functions (to reference), uncertainties, and
+            residuals
+        tdcfs : list
+            list of time dependent correction factors for each measurement
 
         """
         assert ((len(measurement_list) == len(fmin_list) and
@@ -1482,34 +1595,17 @@ class ProcessActuationMeasurement(ProcessMeasurement):
                 (len(measurement_list) == len(fmax_list) and
                  len(fmin_list) == 0))
 
-        ref_date, MAP, ref_spring = self.query_results_from_json(self.meas_type, fmin,
-                                                                 fmax)
-
-        stacked_meas = self.stack_measurements(measurement_list, fmin, fmax,
-                                               fmin_list, fmax_list)
-
-        for idx, _ in enumerate(stacked_meas):
-            gain_tf = MAP[0] * np.ones(len(stacked_meas[idx][0]))
-            if idx == 0:
-                residuals = np.array([stacked_meas[idx][0],
-                                      stacked_meas[idx][1]/gain_tf,
-                                      stacked_meas[idx][2]]).T
-            else:
-                residuals = np.vstack((residuals,
-                                       np.array([stacked_meas[idx][0],
-                                                 stacked_meas[idx][1]/gain_tf,
-                                                 stacked_meas[idx][2]]).T))
-
-        y_pred, sigma, cov = self._gpr(frequencies, residuals,
-                                       RBF_length_scale,
-                                       RBF_length_scale_limits)
+        y_pred, sigma, cov, stacked_meas, tdcfs = self._gpr(
+            frequencies, measurement_list, fmin, fmax,
+            fmin_list, fmax_list, RBF_length_scale,
+            RBF_length_scale_limits, gpr_flim)
 
         # save the result to HDF5
         if save_to_file is not None:
             save_gpr_to_hdf5(save_to_file, self.actuation, self.meas_type,
                              y_pred, cov, frequencies)
 
-        return y_pred, sigma, cov, residuals
+        return y_pred, sigma, cov, stacked_meas, tdcfs
 
 
 def _ln_prob(pars, bound=None):
@@ -1522,25 +1618,36 @@ def _ln_prob(pars, bound=None):
     """
 
     if meas_type_gl == 'sensing':
-        if bound is None:
-            if (pars[2] < 0.0 or pars[2] > 20.0 or pars[3] < 1e-2 or
-                    pars[3] > 1.0):
-                return -np.inf
+        if include_spring_gl:
+            if bound is None:
+                if (pars[2] < 0.0 or pars[2] > 20.0 or pars[3] < 1e-2 or
+                        pars[3] > 1.0):
+                    return -np.inf
+            else:
+                if (pars[0] < bound[0][0] or pars[0] > bound[0][1] or
+                        pars[1] < bound[1][0] or pars[1] > bound[1][1] or
+                        pars[2] < bound[2][0] or pars[2] > bound[2][1] or
+                        pars[3] < bound[3][0] or pars[3] > bound[3][1] or
+                        pars[4] < bound[4][0] or pars[4] > bound[4][1]):
+                    return -np.inf
+
+            this_response = \
+                pars[0] * freqresp(SensingModel.optical_response(
+                    pars[1], pars[2], 1/pars[3], is_pro_spring_gl),
+                    2.0*np.pi*xdata_gl)[1] * np.exp(-2.*np.pi*1j*pars[4] * xdata_gl)
+
         else:
-            if (pars[0] < bound[0][0] or pars[0] > bound[0][1] or
-                    pars[1] < bound[1][0] or pars[1] > bound[1][1] or
-                    pars[2] < bound[2][0] or pars[2] > bound[2][1] or
-                    pars[3] < bound[3][0] or pars[3] > bound[3][1] or
-                    pars[4] < bound[4][0] or pars[4] > bound[4][1]):
-                return -np.inf
-        this_response = (pars[0] *
-                         freqresp(
-                             SensingModel.optical_response(pars[1],
-                                                           pars[2],
-                                                           1/pars[3],
-                                                           is_pro_spring_gl),
-                             2.0*np.pi*xdata_gl)[1] *
-                         np.exp(-2.*np.pi*1j*pars[4] * xdata_gl))
+            if bound is not None:
+                if (pars[0] < bound[0][0] or pars[0] > bound[0][1] or
+                        pars[1] < bound[1][0] or pars[1] > bound[1][1] or
+                        pars[4] < bound[4][0] or pars[4] > bound[4][1]):
+                    return -np.inf
+
+            this_response = \
+                pars[0] * freqresp(SensingModel.optical_response(
+                    pars[1], 0, 1e-3, is_pro_spring_gl),
+                    2.0*np.pi*xdata_gl)[1] * np.exp(-2.*np.pi*1j*pars[4] * xdata_gl)
+
     elif (meas_type_gl == 'actuation_x_arm'
           or meas_type_gl == 'actuation_y_arm'):
         if bound is None:
@@ -1593,7 +1700,7 @@ def _isin(element, test_elements, rel_tol=1e-09, abs_tol=0.0):
     rel_tol : float
         relative tolerance
     abs_tol : float
-        minimum absolute tolerance – useful for comparisons near zero.
+        minimum absolute tolerance - useful for comparisons near zero.
         abs_tol must be at least zero
 
     Returns
