@@ -18,9 +18,11 @@ import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.ticker as plticker
 from matplotlib.backends.backend_pdf import PdfPages
-from pydarm import darm
 from scipy.interpolate import interp1d
 
+from . import darm
+
+
 plt.rcParams.update({'text.usetex': False,
                      'lines.linewidth': 3,
                      'font.family': 'sans-serif',
@@ -130,20 +132,20 @@ class BodePlot:
             self.ax_phase.set_yscale('linear')
             self.ax_phase.set_yticks(self.phase_yticks)
 
-    def legend(self, labels=None):
+    def legend(self, labels=None, **kwargs):
         """add legend
         """
 
         if self.only == '' or self.only == 'mag' or self.only == 'magnitude':
             if labels:
-                self.ax_mag.legend(labels)
+                self.ax_mag.legend(labels, **kwargs)
             else:  # labels can also be passed as kwargs to plot, in which case this is fine
-                self.ax_mag.legend()
+                self.ax_mag.legend(**kwargs)
         if self.only == 'phase':
             if labels:
-                self.ax_mag.legend(labels)
+                self.ax_mag.legend(labels, **kwargs)
             else:
-                self.ax_phase.legend()
+                self.ax_phase.legend(**kwargs)
 
     def save(self, path):
         """save bode plot to file
