@@ -14,9 +14,10 @@
 # You should have received a copy of the GNU General Public License along with
 # pyDARM. If not, see <https://www.gnu.org/licenses/>.
 
+from collections import namedtuple
+
 import numpy as np
 from scipy import signal, io, constants
-from collections import namedtuple
 
 from .utils import serielZPK, compute_digital_filter_response
 from .digital import iopdownsamplingfilters
@@ -317,7 +318,7 @@ class SensingModel(Model):
         # Normalized analog anti-aliasing frequency response
         analog_aa_filter_response = \
             self.analog_aa_or_ai_filter_response(frequencies, idx)
-        norm = np.abs(self.analog_aa_or_ai_filter_response(1e-2,
+        norm = np.abs(self.analog_aa_or_ai_filter_response(np.atleast_1d(1e-2),
                                                            idx))
         analog_aa_filter_response_hi_f = analog_aa_filter_response / norm
 
