@@ -14,16 +14,21 @@
 # You should have received a copy of the GNU General Public License along with
 # pyDARM. If not, see <https://www.gnu.org/licenses/>.
 
-import numpy as np
-import matplotlib.pyplot as plt
-import configparser
 import os
+import configparser
 from copy import deepcopy
-from scipy import signal
+
+import tqdm
 import h5py
+import json
+import numpy as np
+from scipy import signal
+import matplotlib.pyplot as plt
+
 from gwpy.timeseries import TimeSeriesDict as tsd
-from .darm import DARMModel
-from .utils import (read_chain_from_hdf5, read_gpr_from_hdf5)
+
+from .calcs import CALCSModel
+from .utils import (read_chain_from_hdf5, read_gpr_from_hdf5, read_response_curves_from_hdf5)
 from .model import isfloat
 
 
@@ -53,7 +58,7 @@ class DARMUncertainty(object):
         # optional uncertainty will override anything in config
         self._config = None
         self._load_configuration(config)
-        self.darm = DARMModel(config)
+        self.darm = CALCSModel(config)
 
         if uncertainty_config is not None:
             self._load_configuration(uncertainty_config)
@@ -91,6 +96,21 @@ class DARMUncertainty(object):
                 else:
                     self.actuation_gpr_dict[arm][stage] = None
 
+        # In order to account for periods where the uncertainty would
+        # nominally be given by a single measurement, we will multiply the
+        # previous response function curves onto the new draws.
+        # This will inflate the uncertainty as a "stop-gap" measure in
+        # periods where we have few measurements of a new model to
+        # reliably produce an uncertainty estimate from new measurements
+        # alone.
+        if (hasattr(self, 'response_curve_file') and
+                self.response_curve_file != ''):
+            [self.previous_model_response_frequencies,
+             self.previous_model_response_curves] = (
+                read_response_curves_from_hdf5(self.response_curve_file))
+        else:
+            self.response_curve_file = None
+
     def _load_configuration(self, config):
         """Reads configuration and load parameters
 
@@ -116,12 +136,13 @@ class DARMUncertainty(object):
         mcmc_files_dict = {'xarm': {}, 'yarm': {}}
         gpr_files_dict = {'xarm': {}, 'yarm': {}}
         act_delay_sample_dict = {'xarm': {}, 'yarm': {}}
-        cal_line_dict = {'xarm': {}, 'yarm': {}}
         tdcf_channels_dict = {'sensing': {}, 'xarm': {}, 'yarm': {}}
 
         for idx, sec in enumerate(self._config):
             if sec == 'reference-model':
                 setattr(self, 'model', self._config[sec]['model'])
+                setattr(self, 'response_curve_file',
+                        self._config[sec]['previous_model_response_curve_file'])
 
             elif sec == 'sensing-measurement':
                 setattr(self, 'sensing_mcmc_file', self._config[sec]['mcmc'])
@@ -139,12 +160,6 @@ class DARMUncertainty(object):
                     else:
                         act_delay_sample_dict[arm][stage] = value
 
-            # Calibration lines
-            elif 'cal-lines' in sec:
-                arm = ''.join(sec.split('-')[0:2])
-                for n, (key, value) in enumerate(self._config[sec].items()):
-                    cal_line_dict[arm][key] = float(value)
-
             # TDCF channels
             elif sec == 'sensing-tdcf' or sec == 'x-arm-tdcf' or sec == 'y-arm-tdcf':
                 if 'arm' not in sec:
@@ -193,7 +208,6 @@ class DARMUncertainty(object):
         setattr(self, 'actuation_mcmc_files_dict', mcmc_files_dict)
         setattr(self, 'actuation_gpr_files_dict', gpr_files_dict)
         setattr(self, 'act_delay_sample_dict', act_delay_sample_dict)
-        setattr(self, 'cal_line_dict', cal_line_dict)
         setattr(self, 'tdcf_channels_dict', tdcf_channels_dict)
 
     def _channel_list_from_dict(self):
@@ -700,21 +714,20 @@ class DARMUncertainty(object):
 
         # kappa_C and f_cc estimated to compute normalized to reference optical
         # response
-        sensing_line_freq = self.cal_line_dict[
-            f"{self.tdcf_channels_dict['sensing']['pcal_arm'].lower()}arm"
-            ]['sensing']
+        # First get the sensing calibration line frequency
+        sensing_line_freq = self.darm.calcs.cal_line_sens_pcal_frequency
+        # Determine transfer function at the calibration line frequency
         opt_zpk = darm_copy.sensing.optical_response(
             sensing_tdcf[1], darm_copy.sensing.detuned_spring_frequency,
             darm_copy.sensing.detuned_spring_q,
             darm_copy.sensing.is_pro_spring)
         opt = signal.freqresp(opt_zpk, 2*np.pi*sensing_line_freq)[1]
-
+        # Compute the sensing function at the calibration line frequency
         C_CalLineP2 = darm_copy.sensing.compute_sensing(
             np.asarray([sensing_line_freq]))[0]
+        # Divide out the optical response at the calibration line frequency
         EP4 = C_CalLineP2 / opt[0]
 
-        opt *= sensing_tdcf[0]
-
         # TODO: need a way to confirm the EPICS records used to compute the TDCFs
         #       are the same is what is computed here
         EP5 = darm_copy.digital.compute_response(
@@ -723,6 +736,7 @@ class DARMUncertainty(object):
         # the actuator TDCFs above
         A_CalLineP2 = darm_copy.actuation.compute_actuation(
             np.asarray([sensing_line_freq]))[0]
+        # TODO: does this delay still need to be on the following line?
         EP9 = (darm_copy.pcal.compute_pcal_correction(
             np.asarray([sensing_line_freq]))[0] *
             np.exp(-2*np.pi*1j*sensing_line_freq/16384))
@@ -744,10 +758,22 @@ class DARMUncertainty(object):
             this_f_cc = sensing_tdcf[1]
 
         # Scale sensing
-        # TODO: interestingly, the f_cc value is not sampled from the MCMC,
-        #       just the uncertainty of the TDCFs. Consider revisiting this
+        # For optical gain, scale the sampled MCMC value (absolute) by the
+        # sampled TDCF value (relative)
+        # H_c,sample(t) = kappa_c,draw(t) x H_c,draw(0)
+        # where kappa_c can have a central value different than 1.
+        # For cavity pole, the situation is a bit more complicated because we
+        # only have absolute numerical values so we need to construct the
+        # analagous "kappa" value
+        # f_cc,sample(t) = [f_cc,draw(t)/f_cc,MAP] x f_cc,draw(0)
+        # where the ratio in brackets [ ] can have a central value offset from
+        # unity.
+        # Spring frequency, Q, and residual time delay are only sampled from
+        # the MCMC, since we do not have reliable TDCF values
         darm_copy.sensing.coupled_cavity_optical_gain = this_kappa_c * this_sensing[0]
-        darm_copy.sensing.coupled_cavity_pole_frequency = this_f_cc
+        darm_copy.sensing.coupled_cavity_pole_frequency = (
+            (this_f_cc/darm_copy.sensing.coupled_cavity_pole_frequency) *
+            this_sensing[1])
         darm_copy.sensing.detuned_spring_frequency = this_sensing[2]
         darm_copy.sensing.detuned_spring_q = this_sensing[3]
         darm_copy.sensing.single_pole_approximation_delay_correction += \
@@ -924,7 +950,8 @@ class DARMUncertainty(object):
 
     def compute_response_uncertainty(self, gpstime, frequencies, trials=1000,
                                      data=None, shift_sample_tf=None,
-                                     save_to_file=None):
+                                     save_to_file_h5=None, save_to_file_txt=None,
+                                     seed=None):
         """
         This method samples the response function uncertainty a number of times
         in order to obtain an estimate of the systematic error and associated
@@ -952,8 +979,15 @@ class DARMUncertainty(object):
             Multiply sampled responses by a transfer function. The transfer
             function needs to be computed from the same frequenices as this
             method uses otherwise results are unpredictable
-        save_to_file : `str`, optional
-            Path to file saved as HDF5 (.h5 or .hdf5) or ASCII (.txt)
+        save_to_file_h5 : `str`, optional
+            Path to file for saving response curves and samples as
+            HDF5 (.h5 or .hdf5)
+        save_to_file_txt : `str`, optional
+            Path to file for saving uncertainty envelope as ASCII (.txt)
+        seed: `int`, optional
+            Seed for the random generation of the uncertainty envelope samples.
+            The default value (None) will mean results are not reproducible,
+            whereas an integer value will return reproducible results.
 
         Returns
         -------
@@ -961,9 +995,20 @@ class DARMUncertainty(object):
             list of response transfer function samples
         """
 
+        np.random.seed(seed)
         nominal_response, data = self.nominal_response(gpstime, frequencies,
                                                        data)
 
+        # Precompute the suspension filters at particularly relevant frequencies
+        sensing_line_freq = self.darm.calcs.cal_line_sens_pcal_frequency
+
+        if 'actuation_x_arm' in self._config:
+            _ = self.darm.actuation.xarm.sus_digital_filters_response(frequencies)
+            _ = self.darm.actuation.xarm.sus_digital_filters_response(sensing_line_freq)
+        if 'actuation_y_arm' in self._config:
+            _ = self.darm.actuation.yarm.sus_digital_filters_response(frequencies)
+            _ = self.darm.actuation.yarm.sus_digital_filters_response(sensing_line_freq)
+
         # Store data for each of the samples so that it can be saved to an
         # HDF5 file
         response_samples = np.zeros((trials, len(frequencies)), dtype='complex128')
@@ -979,13 +1024,20 @@ class DARMUncertainty(object):
                                                           dtype='complex128')
 
         # Loop over the number of trials
-        for n in range(trials):
+        for n in tqdm.tqdm(range(trials)):
             (response, data, sens_pars, act_pars, sensing_syserr_samples[n, :],
              act_syserr_dict) = self.sample_response(
                  gpstime, frequencies, data=data)
 
             response_samples[n, :] = response / nominal_response
 
+            # Multiply in response curves from previous measurements
+            if self.response_curve_file and \
+                    self.response_curve_file != '':
+                response_samples[n, :] *= np.interp(
+                    frequencies, self.previous_model_response_frequencies,
+                    self.previous_model_response_curves[n, :])
+
             if n == 0:
                 sensing_pars_samples = np.zeros((trials, len(sens_pars)))
 
@@ -1006,12 +1058,12 @@ class DARMUncertainty(object):
         # TODO: do we need to use astropy.table.Table in HDF5? The O3 version used it
         # TODO: what else is required to be saved into these files? The O3
         #       version had a lot but it wasn't clear if everything was needed
-        if save_to_file is not None:
-            if (os.path.splitext(save_to_file) == '.h5' or
-                    os.path.splitext(save_to_file) == '.hdf5'):
-                save_to_file = os.path.normpath(save_to_file)
-                with h5py.File(save_to_file, 'r') as f:
-                    f.create_dataset('config', data=self._config)
+        if (save_to_file_h5 is not None):
+            if ('.h5' in os.path.splitext(save_to_file_h5) or
+                    '.hdf5' in os.path.splitext(save_to_file_h5)):
+                save_to_file_h5 = os.path.normpath(save_to_file_h5)
+                with h5py.File(save_to_file_h5, 'w') as f:
+                    f.create_dataset('config', data=json.dumps(dict(self._config._sections)))
                     f.create_dataset('deltaR/freq', data=frequencies)
                     f.create_dataset('deltaR/draws', data=response_samples)
                     f.create_dataset('CalParams/sensing', data=sensing_pars_samples)
@@ -1024,8 +1076,13 @@ class DARMUncertainty(object):
                         for j, stage in enumerate(act_syserr_samples[arm].keys()):
                             f.create_dataset(f'GPR/{arm}/{stage}',
                                              data=act_syserr_samples[arm][stage])
-                    f.create_dataset('shift_sample_tf', data=shift_sample_tf)
-            elif (os.path.splitext(save_to_file) == '.txt'):
+                    if shift_sample_tf:
+                        f.create_dataset('shift_sample_tf', data=shift_sample_tf)
+                    else:
+                        f.create_dataset('shift_sample_tf', data='None')
+
+        if save_to_file_txt is not None:
+            if ('.txt' in os.path.splitext(save_to_file_txt)):
                 response_mag_quant, response_pha_quant = \
                     self.response_quantiles(response_samples)
                 save_txt_response = np.vstack((
@@ -1034,10 +1091,10 @@ class DARMUncertainty(object):
                     response_mag_quant[0, :], response_pha_quant[0, :],
                     response_mag_quant[2, :], response_pha_quant[2, :])).T
                 header = "{:12} {:21} {:21} {:21} {:21} {:21} {:21}".format(
-                    'Frequency', 'Median mag', 'Phase (Rad)',
+                    'Frequency', 'Median mag', 'Median phase (Rad)',
                     '16th percentile mag', '16th percentile phase',
                     '84th percentile mag', '84th percentile phase')
-                np.savetxt(save_to_file, save_txt_response, fmt='%+1.7e', header=header)
+                np.savetxt(save_to_file_txt, save_txt_response, fmt='%+1.7e', header=header)
 
         return response_samples
 
@@ -1077,13 +1134,9 @@ class DARMUncertainty(object):
 
     @staticmethod
     def plot_response_samples(frequencies, RRMag, RRPha,
-                              response_mag_quant, response_pha_quant,
+                              response_mag_quant, response_pha_quant, ifo,
                               filename=None):
         """
-        Placeholder plotting for response function samples
-
-        TODO: replace this with standard pyDARM plotting
-
         Parameters
         ----------
         frequencies : `float`, array-like
@@ -1091,28 +1144,71 @@ class DARMUncertainty(object):
         RRPha : `float`, list of arrays
         response_mag_quant : `float`, array-like
         response_pha_quant : `float`, array-like
+        ifo: `str`, LHO or LLO
 
         Returns
         -------
+        fig: `matplotlib.figure.Figure`, figure of uncertainty
         """
+        import matplotlib as mpl
+        mpl.rcParams.update({'text.usetex': True,
+                             'axes.linewidth': 1,
+                             'axes.grid': True,
+                             'axes.labelweight': 'normal',
+                             'font.family': 'DejaVu Sans',
+                             'font.size': 20})
+        if ifo == 'LHO':
+            ifo_color = 'r'
+        else:
+            ifo_color = 'b'
 
-        h = plt.figure(figsize=(20, 15))
-        f1 = h.add_subplot(211)
-        f2 = h.add_subplot(212)
-        legendLabel = '50 p/m 68/2 %'
-        for i in range(len(RRMag)):
-            f1.semilogx(frequencies, 100 * (RRMag[i, :]-1.0))
-            f2.semilogx(frequencies, 180 / np.pi * (RRPha[i, :]))
-        for j in range(len(response_mag_quant)):
-            f1.semilogx(frequencies, 100 * (response_mag_quant[j, :]-1.0), label=legendLabel)
-            f2.semilogx(frequencies, 180 / np.pi * (response_pha_quant[j, :]), label=legendLabel)
-        f1.set_xlabel('Frequency [Hz]')
-        f2.set_xlabel('Frequency [Hz]')
-        f1.set_xlim(10, 2000)
-        f2.set_xlim(10, 2000)
-        f1.set_ylabel('Magnitude')
-        f2.set_ylabel('Phase [deg.]')
-        f1.set_ylim(-10, 10)
-        f2.set_ylim(-10, 10)
+        freqRange = [10.0, 5000.0]
+        yMagRange = [-10.0, 10.0]
+        yPhaseRange = [-10.0, 10.0]
+
+        fig, axes = plt.subplots(nrows=2, ncols=1, figsize=(15, 12))
+        ax0, ax1 = axes.flat
+
+        ax0.plot(frequencies, 100 * (response_mag_quant[1, :] - 1.0),
+                 color=ifo_color, label=r'Median')
+        ax0.plot(frequencies, 100 * (response_mag_quant[0, :] - 1.0),
+                 linestyle='--', color=ifo_color, label=r'$1 \sigma$ Uncertainty')
+        ax0.plot(frequencies, 100 * (response_mag_quant[2, :] - 1.0),
+                 linestyle='--', color=ifo_color)
+        ax0.fill_between(frequencies, 100 * (response_mag_quant[0, :] - 1.0),
+                         100 * (response_mag_quant[2, :] - 1.0), color=ifo_color, alpha=0.3)
+        ax0.set_xlim(freqRange)
+        ax0.set_ylim(yMagRange)
+        ax0.minorticks_on()
+        ax0.set_yticks(np.arange(yMagRange[0], yMagRange[1]+1, 2))
+        ax0.grid(ls='-', which='major', lw=1.1)
+        ax0.grid(ls='--', which='minor')
+
+        ax1.plot(frequencies, 180/np.pi*response_pha_quant[1, :], color=ifo_color, label=r'Median')
+        ax1.plot(frequencies, 180/np.pi*response_pha_quant[0, :], linestyle='--',
+                 color=ifo_color, label=r'$1 \sigma$ Uncertainty')
+        ax1.plot(frequencies, 180/np.pi*response_pha_quant[2, :], linestyle='--', color=ifo_color)
+        ax1.fill_between(frequencies, 180/np.pi*response_pha_quant[0, :],
+                         180/np.pi*response_pha_quant[2, :], color=ifo_color, alpha=0.3)
+        ax1.set_xlim(freqRange)
+        ax1.set_ylim(yPhaseRange)
+        ax1.minorticks_on()
+        ax1.set_yticks(np.arange(yMagRange[0], yMagRange[1]+1, 2))
+        ax1.grid(ls='-', which='major', lw=1.1)
+        ax1.grid(ls='--', which='minor')
+
+        for ax in axes.flat:
+            ax.set_xscale("log")
+            ax.set_xticks(np.array([20, 50, 100, 200, 500, 1000, 2000, 5000]))
+            ax.set_xticklabels(['20', '50', '100', '200', '500', '1000', '2000', '5000'])
+            legend = ax.legend(loc='lower center', ncol=2, handlelength=3)
+            for i in legend.legendHandles:
+                i.set_linewidth(2)
+
+        ax1.set_xlabel(r'Frequency [Hz]')
+        ax0.set_ylabel(r'$|R^{\rm (sample)}/R^{\rm (MAP)}|-1$ [\%]')
+        ax1.set_ylabel(r'$\angle[R^{\rm (sample)}/R^{\rm (MAP)}]$ [deg]')
         if filename is not None:
-            plt.savefig(filename, bbox_inches='tight')
+            fig.savefig(filename, bbox_inches='tight', pad_inches=0.2)
+
+        return fig
