@@ -14,13 +14,17 @@
 # You should have received a copy of the GNU General Public License along with
 # pyDARM. If not, see <https://www.gnu.org/licenses/>.
 
+import re
+from os.path import normpath
+from collections import namedtuple
+
+import h5py
 from scipy import signal
 import numpy as np
 from numpy.polynomial import polynomial as P
-from collections import namedtuple
-import re
-import h5py
-from os.path import normpath
+
+import epics
+
 
 cds_filter = namedtuple('cds_filter', ['name', 'soscoef', 'fs', 'design'])
 cds_filter_bank = namedtuple('cds_filter_bank',
@@ -559,8 +563,8 @@ def save_chain_to_hdf5(filename, meas_model, fmin, fmax, measurement, chain):
     ----------
     filename : str
         Output filename
-    meas_model : str
-        Configuration string for the model
+    meas_model : `str` or `dict`
+        Configuration string or dictionary for the model
     fmin : float
         Minimum frequency used in the MCMC fit
     fmax : float
@@ -578,8 +582,6 @@ def save_chain_to_hdf5(filename, meas_model, fmin, fmax, measurement, chain):
         f.create_dataset('measurement', data=str(measurement))
         f.create_dataset('posteriors', data=chain)
 
-    return
-
 
 def read_chain_from_hdf5(filename):
     """
@@ -682,6 +684,32 @@ def read_gpr_from_hdf5(filename, measurement):
     return meas_model, y_pred, cov, frequencies
 
 
+def read_response_curves_from_hdf5(filename):
+    """
+    Read the residual response curves from the response curve uncertainty envelope calculation
+
+    Parameters
+    ----------
+    filename : str
+        Response curve file
+
+    Returns
+    -------
+
+    frequencies : `float', array-like
+        Array of frequencies for the response curves
+    response_curves : `complex', array-like
+        Array of individual residual response function curves
+    """
+    with h5py.File(normpath(filename), 'r') as f:
+
+        data = f.get('deltaR')
+        frequencies = np.array(data.get('freq')[()], dtype=float)
+        response_curves = np.array(data.get('draws')[()], dtype=complex)
+
+    return frequencies, response_curves
+
+
 def read_eta_or_syserr_from_hdf5(filename, measurement=None, sensing=False, actuation=False):
     """
     Read eta results(See aLOG:62621) or sensing and actuation systematic error defined by user
@@ -754,3 +782,40 @@ def thiran_delay_filter(tau, Ts):
     aa = a[::-1]
     sys = signal.TransferFunction(aa, a, dt=Ts)
     return sys
+
+
+def write_dict_epics(dictionary, dry_run=True, IFO=None, save_file=None):
+    """Write dictionary of key/value pairs to EPICS records (keys are channel names)
+
+    Parameters
+    ----------
+    dictionary : dict
+        the dictionary of values to write
+    dry_run : boolean
+        actually write to EPICS if value is False
+    IFO : str
+        optional IFO string for IFO channel prefix if needed
+    save_file: path
+        optional file to save written record values
+
+    """
+    if save_file:
+        f = open(save_file, 'w')
+    for channel, value in dictionary.items():
+        if IFO:
+            IFO = IFO.upper()
+            channel = f"{IFO}:{channel}"
+        value = np.float32(value)
+        print(f"{channel} = {value}")
+        if not dry_run:
+            epics.caput(channel, f'{value:0.6e}', wait=True, timeout=1)
+            try:
+                test_value = epics.caget(channel)
+                if not np.isclose(test_value, value):
+                    raise RuntimeError(f'Channel write did not return expected result: {channel} should be {value}, returned {test_value}') # noqa E501
+            except TypeError:
+                raise RuntimeError(f'Unable to validate channel write: {channel}')
+        # write out value only if it was successfully written to EPICS
+        if save_file:
+            f.write(f'{channel} {value}\n')
+    f.close()
