@@ -1,6 +1,7 @@
 import unittest
 import pydarm
 import numpy as np
+import math
 import h5py
 import os
 from pydarm.FIR import (FIRfilter, createFIRfilter,
@@ -11,44 +12,67 @@ from pydarm.FIR import (FIRfilter, createFIRfilter,
 class TestFIRfilter(unittest.TestCase):
 
     def setUp(self):
-        self.window = np.array([6.747975786659215006e-03,
-                                2.767078112501164186e-02,
-                                7.377080623487733413e-02,
-                                1.551839702572553048e-01,
-                                2.768990448483386047e-01,
-                                4.345680722833729082e-01,
-                                6.125095782158316293e-01,
-                                7.853264037228884220e-01,
-                                9.232791494417179612e-01,
-                                1.000000000000000000e+00,
-                                1.000000000000000000e+00,
-                                9.232791494417179612e-01,
-                                7.853264037228884220e-01,
-                                6.125095782158316293e-01,
-                                4.345680722833729082e-01,
-                                2.768990448483386047e-01,
-                                1.551839702572553048e-01,
-                                7.377080623487733413e-02,
-                                2.767078112501164186e-02,
-                                6.747975786659215006e-03])
+        self.advance_array = np.array([ 1.+0.00000000e+00j, -1.+1.22464680e-16j,  1.-2.44929360e-16j,
+                                        -1.+3.67394040e-16j,  1.-4.89858720e-16j, -1.+6.12323400e-16j,
+                                         1.-7.34788079e-16j, -1.+8.57252759e-16j,  1.-9.79717439e-16j])
+        self.delay_array = np.array([ 1.+0.00000000e+00j, -1.-1.22464680e-16j,  1.+2.44929360e-16j,
+                                     -1.-3.67394040e-16j,  1.+4.89858720e-16j, -1.-6.12323400e-16j,
+                                      1.+7.34788079e-16j, -1.-8.57252759e-16j,  1.+9.79717439e-16j])
+        self.delay_samples = 8
+        self.df = 1.0
+        self.dt = 0.0625
+        self.freq_array = np.array([0., 1., 2., 3., 4., 5., 6., 7., 8.])
+        self.latency = 0.5
+        self.samples_to_HPcorner = 1
+        self.samples_to_LPcorner = 0
+        self.window = np.array([0.00233883, 0.02623098, 0.09449865, 0.22677684, 0.42285187,
+                       0.65247867, 0.85980208, 0.98349415, 0.98349415, 0.85980208,
+                       0.65247867, 0.42285187, 0.22677684, 0.09449865, 0.02623098,
+                       0.00233883])
 
     def tearDown(self):
         del self.window
+        del self.advance_array
+        del self.delay_array
+        del self.delay_samples
+        del self.df
+        del self.dt
+        del self.freq_array
+        del self.latency
+        del self.samples_to_HPcorner
+        del self.samples_to_LPcorner
 
     def test_FIRfilter(self):
-        test_filter = FIRfilter(fNyq=10, dur=1, highpass_fcut=10, lowpass_fcut=None,
-                                latency=None, window_type='dpss', freq_res=3.0)
-        self.assertEqual(len(test_filter.window), len(self.window))
+        test_FIRfilter = FIRfilter(fNyq=8, dur=1, highpass_fcut=4, lowpass_fcut=None,
+                                latency=None, window_type='kaiser', freq_res=3.0)
+
+        self.assertEqual(test_FIRfilter.delay_samples, self.delay_samples)
+        self.assertEqual(test_FIRfilter.df, self.df)
+        self.assertEqual(test_FIRfilter.dt, self.dt)
+        self.assertEqual(test_FIRfilter.latency, self.latency)
+        self.assertEqual(test_FIRfilter.samples_to_HPcorner, self.samples_to_HPcorner)
+        self.assertEqual(test_FIRfilter.samples_to_LPcorner, self.samples_to_LPcorner)
+        self.assertEqual(len(test_FIRfilter.advance_array), len(self.advance_array))
+        for n in range(len(self.advance_array)):
+            self.assertAlmostEqual(np.abs(test_FIRfilter.advance_array[n]) / np.abs(self.advance_array[n]), 1.0)
+            self.assertAlmostEqual(np.angle(test_FIRfilter.advance_array[n], deg=True) - np.angle(self.advance_array[n], deg=True), 0.0)
+        self.assertEqual(len(test_FIRfilter.delay_array), len(self.delay_array))
+        for n in range(len(self.delay_array)):
+            self.assertAlmostEqual(np.abs(test_FIRfilter.delay_array[n]) / np.abs(self.delay_array[n]), 1.0)
+            self.assertAlmostEqual(np.angle(test_FIRfilter.delay_array[n], deg=True) - np.angle(self.delay_array[n], deg=True), 0.0)
+        self.assertEqual(len(test_FIRfilter.freq_array), len(self.freq_array))
+        for n in range(len(self.freq_array)):
+            self.assertAlmostEqual(test_FIRfilter.freq_array[n], self.freq_array[n])
+        self.assertEqual(len(test_FIRfilter.window), len(self.window))
         for n in range(len(self.window)):
-            self.assertAlmostEqual(np.real(test_filter.window[n]),
-                                   np.real(self.window[n]))
+            self.assertAlmostEqual(test_FIRfilter.window[n] / self.window[n], 1.0, places=5)
 
 
 class TestCreateFIRfilter(unittest.TestCase):
 
     def setUp(self):
-        self.FIRPars = FIRfilter(fNyq=10, dur=1, highpass_fcut=10, lowpass_fcut=None,
-                                 latency=None, window_type='dpss', freq_res=3.0)
+        self.FIRPars = FIRfilter(fNyq=8, dur=1, highpass_fcut=4, lowpass_fcut=None,
+                                 latency=None, window_type='kaiser', freq_res=3.0)
         self.config = '''
 [metadata]
 [interferometer]
@@ -80,17 +104,10 @@ omc_filter_gain = 1, 1
 omc_front_end_trans_amplifier_compensation = ON, ON
 omc_front_end_whitening_compensation_mode1 = ON, ON
 '''
-        self.known_Cfir = np.array(
-            [-1227.5838658444798, 3076.9909960599016,
-             8947.14656965102, -44153.12903009791,
-             41690.73653592548, 100767.29509196931,
-             -304463.1208524438, 201108.6390562277,
-             475074.27072160656, -1368003.356637503,
-             1709874.866117381, -1187997.4667580684,
-             315569.9399449083, 208373.70618745242,
-             -228827.46425164823, 62660.986752700665,
-             23681.058343631747, -19301.994938787342,
-             2292.852371773494, 954.6954319571059])
+        self.known_Cfir = np.array([-6.63850419e+01,  8.93362543e+02,  8.56077110e+04, -2.27887893e+05,
+                                    -8.86759535e+05,  1.61591927e+06,  1.23433242e+06, -2.69276290e+06,
+                                     1.79904168e+06, -1.74942704e+06,  6.65554086e+05,  9.35475644e+05,
+                                    -4.10502392e+05, -8.87882831e+04,  2.10864177e+04,  1.42418753e+02])
 
     def tearDown(self):
         del self.FIRPars
@@ -100,19 +117,18 @@ omc_front_end_whitening_compensation_mode1 = ON, ON
     def test_createFIRfilter(self):
         C = pydarm.sensing.SensingModel(self.config)
         Cf = C.compute_sensing(self.FIRPars.freq_array)
-        test_Cfir = createFIRfilter(self.FIRPars, Cf)[0]
+        test_Cfir, test_Cmodel = createFIRfilter(self.FIRPars, Cf)
 
         for n in range(len(self.known_Cfir)):
-            self.assertAlmostEqual(np.real(test_Cfir[n]) /
-                                   np.real(self.known_Cfir[n]),
+            self.assertAlmostEqual(test_Cfir[n] /
+                                   self.known_Cfir[n],
                                    1.0)
 
-
 class TestCorrectFIRfilter(unittest.TestCase):
 
     def setUp(self):
-        self.FIRPars = FIRfilter(fNyq=10, dur=1, highpass_fcut=10, lowpass_fcut=None,
-                                 latency=None, window_type='dpss', freq_res=3.0)
+        self.FIRPars = FIRfilter(fNyq=8, dur=1, highpass_fcut=4, lowpass_fcut=None,
+                                 latency=None, window_type='kaiser', freq_res=3.0)
         self.config = '''
 [metadata]
 [interferometer]
@@ -144,17 +160,15 @@ omc_filter_gain = 1, 1
 omc_front_end_trans_amplifier_compensation = ON, ON
 omc_front_end_whitening_compensation_mode1 = ON, ON
 '''
-        self.known_Corfir = np.array(
-            [-169801.88070366555-219.1204830342667j,
-             -957689.5963380174-34613.22973139465j,
-             -4098624.0961085237-351429.94821935653j,
-             -21990666.91293493-3485607.618296178j,
-             26602599.411241055-548641.5068307472j,
-             11374028.314446637+243703.54819104425j,
-             6989957.85280983+5347.050013464119j,
-             4675314.739251457-196822.09592547035j,
-             4268072.935259017-178027.61932502355j,
-             4018006.170981767-172823.8165376102j])
+
+        self.known_Corfir = np.array([-169801.88070367-2.19120483e+02j,
+                               -698906.37183737-2.74819393e+04j,
+                              -2030852.42623273+8.40205385e+03j,
+                             -54502988.18785564+4.75790494e+07j,
+                              47217941.0142496 +8.94536006e+06j,
+                               6654278.33719367-1.81089469e+05j,
+                               4974433.40023975-4.73425087e+05j,
+                               4662225.0011754 -2.41147877e+05j])
 
     def tearDown(self):
         del self.FIRPars
@@ -166,6 +180,8 @@ omc_front_end_whitening_compensation_mode1 = ON, ON
         Cf = C.compute_sensing(self.FIRPars.freq_array)
         test_Cfir = createFIRfilter(self.FIRPars, Cf)[0]
         test_Corfir = correctFIRfilter(self.FIRPars, test_Cfir, Cf,  [2, 4, 7, 9])
+        # The first element is zero so we will delete this one from the check
+        # to avoid divide by zero errors
         test_Corfir = np.delete(test_Corfir, 0)
 
         self.assertEqual(len(test_Corfir), len(self.known_Corfir))
@@ -209,8 +225,8 @@ class Testtwo_tap_zero_filter(unittest.TestCase):
 class Testcheck_td_vs_fd(unittest.TestCase):
 
     def setUp(self):
-        self.FIRPars = FIRfilter(fNyq=10, dur=1, highpass_fcut=10, lowpass_fcut=None,
-                                 latency=None, window_type='hann', freq_res=3.0)
+        self.FIRPars = FIRfilter(fNyq=2048, dur=1, highpass_fcut=8, lowpass_fcut=None,
+                                 latency=None, window_type='kaiser', freq_res=3.0)
         self.config = '''
 [metadata]
 [interferometer]
@@ -242,49 +258,38 @@ omc_filter_gain = 1, 1
 omc_front_end_trans_amplifier_compensation = ON, ON
 omc_front_end_whitening_compensation_mode1 = ON, ON
 '''
-        self.known_freq_array = np.array(
-            [0., 0.125, 0.25, 0.375, 0.5, 0.625, 0.75,
-             0.875, 1., 1.125, 1.25, 1.375, 1.5, 1.625,
-             1.75, 1.875, 2., 2.125, 2.25, 2.375, 2.5,
-             2.625, 2.75, 2.875, 3., 3.125, 3.25, 3.375,
-             3.5, 3.625, 3.75, 3.875, 4., 4.125, 4.25,
-             4.375, 4.5, 4.625, 4.75, 4.875, 5., 5.125,
-             5.25, 5.375, 5.5, 5.625, 5.75, 5.875, 6.,
-             6.125, 6.25, 6.375, 6.5, 6.625, 6.75, 6.875,
-             7., 7.125, 7.25, 7.375, 7.5, 7.625, 7.75,
-             7.875, 8., 8.125, 8.25, 8.375, 8.5, 8.625,
-             8.75, 8.875, 9., 9.125, 9.25, 9.375, 9.5,
-             9.625, 9.75, 9.875, 10.])
 
     def tearDown(self):
         del self.FIRPars
         del self.config
-        del self.known_freq_array
 
     def test_check_td_vs_fd(self):
         C = pydarm.sensing.SensingModel(self.config)
         Cf = C.compute_sensing(self.FIRPars.freq_array)
-        test_Cfir = createFIRfilter(self.FIRPars, Cf)
-        test_ctvf = check_td_vs_fd(test_Cfir[0], Cf, fNyq=10,
-                                   delay_samples=self.FIRPars.delay_samples,
-                                   filename="res_corr_fd_comparison.png",
-                                   plot_title="Residual corrections comparison.")
-        test_freq = test_ctvf[0][0]
+        test_Cfir = createFIRfilter(self.FIRPars, Cf)[0]
+        ctvf = check_td_vs_fd(test_Cfir, Cf, fNyq=self.FIRPars.fNyquist,
+                                   delay_samples=self.FIRPars.delay_samples)
 
-        # Note that the 0th element == 0 so we test differently 
-        for n in range(len(test_freq)):
-            if n == 0:
-                self.assertAlmostEqual(test_freq[n], self.known_freq_array[n])
-            else:
-                self.assertAlmostEqual(test_freq[n] / self.known_freq_array[n],
-                                       1, places=6)
+        test_freq = ctvf[0][0]
+        test_mag_ratios = ctvf[1][0]
+        test_phase_diffs = ctvf[2][0]
+
+        df = test_freq[1] - test_freq[0]
+        # For this configuration, check that the magnitude and phase are close to
+        # 1 and 0, respectively between 50 and 1000 Hz
+        Nmin = math.ceil(40 / df)
+        Nmax = math.ceil(1000 / df)
+ 
+        for n in range(Nmin, Nmax):
+            self.assertLess(np.abs(test_mag_ratios[n]), 1.01)
+            self.assertLess(np.abs(test_phase_diffs[n]), 1)
 
 
 class TestCheckTdVsFdResponse(unittest.TestCase):
 
     def setUp(self):
-        self.FIRPars = FIRfilter(fNyq=10, dur=1, highpass_fcut=10, lowpass_fcut=None,
-                                 latency=None, window_type='hann', freq_res=3.0)
+        self.FIRPars = FIRfilter(fNyq=2048, dur=1, highpass_fcut=8, lowpass_fcut=None,
+                                 latency=None, window_type='kaiser', freq_res=3.0)
         self.config = '''
 [metadata]
 [interferometer]
@@ -392,61 +397,53 @@ anti_aliasing_rate_string = 16k
 anti_aliasing_method      = biquad
 analog_anti_aliasing_file = test/H1aa.mat
 '''
-        self.known_freq_array = np.array(
-            [0.0, 0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0, 2.25, 2.5, 2.75,
-             3.0, 3.25, 3.5, 3.75, 4.0, 4.25, 4.5, 4.75, 5.0, 5.25, 5.5, 5.75,
-             6.0, 6.25, 6.5, 6.75, 7.0, 7.25, 7.5, 7.75, 8.0, 8.25, 8.5, 8.75,
-             9.0, 9.25, 9.5, 9.75, 10.0])
-
     def tearDown(self):
         del self.FIRPars
         del self.config
-        del self.known_freq_array
 
-    def test_check_td_vs_fd(self):
+    def test_check_td_vs_fd_response(self):
         freq = self.FIRPars.freq_array
+        # DARM model throws warnings when computed at 0 Hz, so remove that from frequency array
         freq = np.delete(freq, 0)
         darm = pydarm.darm.DARMModel(self.config)
-        InvC = 1/darm.sensing.compute_sensing(freq)
-        TST_filt = darm.actuation.xarm.compute_actuation_single_stage(freq, stage='TST')
-        PUM_filt = darm.actuation.xarm.compute_actuation_single_stage(freq, stage='PUM')
-        UIM_filt = darm.actuation.xarm.compute_actuation_single_stage(freq, stage='UIM')
-        dig_filt = darm.digital.compute_response(freq)
-        R = darm.compute_response_function(freq)
+        # To get frequency vectors correct, need to reinsert the zero frequency component after DARM components are computed
+        # Just set this DC component to 0
+        InvC = np.insert(1/darm.sensing.compute_sensing(freq), 0, 0)
+        TST = np.insert(darm.actuation.xarm.compute_actuation_single_stage(freq, stage='TST'), 0, 0)
+        PUM = np.insert(darm.actuation.xarm.compute_actuation_single_stage(freq, stage='PUM'), 0, 0)
+        UIM = np.insert(darm.actuation.xarm.compute_actuation_single_stage(freq, stage='UIM'), 0, 0)
+        D = np.insert(darm.digital.compute_response(freq), 0, 0)
+        R = np.insert(darm.compute_response_function(freq), 0, 0)
 
-        # TODO: this function emits a lot of warnings. Experts need to look at
-        # this more carefully
-        test_ctvf = check_td_vs_fd_response(InvC,
+        InvC_filt = createFIRfilter(self.FIRPars, InvC)[0]
+        TST_filt = createFIRfilter(self.FIRPars, TST)[0]
+        PUM_filt = createFIRfilter(self.FIRPars, PUM)[0]
+        UIM_filt = createFIRfilter(self.FIRPars, UIM)[0]
+
+        test_freq, test_ratio_mag, test_ratio_pha = check_td_vs_fd_response(InvC_filt,
                                             None,
                                             TST_filt,
                                             PUM_filt,
                                             UIM_filt,
                                             None,
-                                            dig_filt,
+                                            D,
                                             R,
-                                            invsens_fNyq=10,
-                                            invsens_highpass_fNyq=10,
-                                            act_fNyq=10,
-                                            D_fNyq=10,
-                                            R_fNyq=10,
-                                            invsens_delay=None,
-                                            invsens_highpass_delay=None,
-                                            act_delay=None,
-                                            act_highpass_delay=None,
-                                            time_delay=1.0/16384,
-                                            filename="td_vs_fd_response.png",
-                                            plot_title="Response Function",
-                                            legend=['DARM model', 'FIR filters'])
-        test_freq = test_ctvf[0]
-        test_ratio_mag = test_ctvf[1]
-        test_ratio_pha = test_ctvf[2]
-
-        # TODO: need to add test_ratio_mag and test_ratio_pha here.
-        # Evan is concerned these don't look good right now - they are not
-        # close to 1 in magnitude and 0 degrees phase
-        for n in range(len(test_freq)):
-            self.assertAlmostEqual(test_freq[n], self.known_freq_array[n])
+                                            time_delay = 0.0,
+                                            invsens_fNyq=self.FIRPars.fNyquist,
+                                            act_fNyq=self.FIRPars.fNyquist,
+                                            D_fNyq=self.FIRPars.fNyquist,
+                                            R_fNyq=self.FIRPars.fNyquist)
 
+        
+        df = test_freq[1] - test_freq[0]
+        # For this configuration, check that the magnitude and phase are close to
+        # 1 and 0, respectively between 50 and 1000 Hz
+        Nmin = math.ceil(40 / df)
+        Nmax = math.ceil(1000 / df)
+        
+        for n in range(Nmin, Nmax):
+            self.assertLess(np.abs(test_ratio_mag[n]), 1.05)
+            self.assertLess(np.abs(test_ratio_pha[n]), 5)
 
 class TestGDS_FIR_filter_generation(unittest.TestCase):
     def setUp(self):
@@ -521,12 +518,12 @@ class TestGDS(unittest.TestCase):
     def test_GDS(self):
         config = './example_model_files/H1_20190416.ini'
         FG = pydarm.FIR.FilterGeneration(config)
-        FG.GDS(ctrl_window_type='dpss', res_window_type='dpss',
+        FG.GDS(ctrl_window_type='kaiser', res_window_type='kaiser',
                make_plot=False, output_filename='./test/GDS.npz',
                plots_directory='./examples/GDS_plots')
         gds = np.load('./test/GDS.npz')
         ctrl_td = gds['ctrl_corr_filter']
-        for n in range(1000, len(self.ctrl_corr_td)-1000):
+        for n in range(len(self.ctrl_corr_td)):
             self.assertAlmostEqual(abs((self.ctrl_corr_td[n] / ctrl_td[n])
                                        - 1), 0, places=3)
 
@@ -546,12 +543,12 @@ class TestDCS(unittest.TestCase):
     def test_DCS(self):
         config = './example_model_files/H1_20190416.ini'
         FG = pydarm.FIR.FilterGeneration(config)
-        FG.DCS(act_window_type='dpss', invsens_window_type='dpss',
+        FG.DCS(act_window_type='kaiser', invsens_window_type='kaiser',
                make_plot=False, output_filename='./test/DCS.npz',
                plots_directory='examples/DCS_plots')
-        gds = np.load('./test/DCS.npz')
-        act_tst = gds['actuation_tst']
-        for n in range(1000, len(self.known_act_tst)-1000):
+        dcs = np.load('./test/DCS.npz')
+        act_tst = dcs['actuation_tst']
+        for n in range(len(self.known_act_tst)):
             self.assertAlmostEqual(abs((self.known_act_tst[n] / act_tst[n])
                                        - 1), 0, places=3)
 
