@@ -1473,12 +1473,24 @@ darm_feedback_y = OFF, ON, ON, ON
              7.665512735370611e-20+3.621744507461856e-19j,
              -1.4601630153846298e-20-5.34526407669915e-20j,
              4.777143021044974e-21-7.734375898626229e-22j])
+        self.known_super_act_drivealign = 2*np.array(
+            [2.222881500981626e-15-3.691011841669097e-16j,
+             -1.1764193182970055e-15-2.5849635956379406e-16j,
+             -3.1617534435707367e-16+9.92839053775603e-16j,
+             1.1700305084971196e-16+2.1853458973419276e-16j,
+             5.006230661758991e-17+2.4902470964105e-17j,
+             9.308653195938007e-18-1.9222273202986028e-19j,
+             1.1111368291657706e-18-5.936117452101705e-19j,
+             5.412995749774128e-20-1.7611360028653088e-19j,
+             -2.4897361921808887e-20+3.511338429355629e-21j,
+             -1.3890965613664794e-21-1.7923601565333542e-21j])
 
     def tearDown(self):
         del self.frequencies
         del self.model_string
         del self.known_A
         del self.known_super_act
+        del self.known_super_act_drivealign
 
     def test_multi_actuation(self):
         A = pydarm.actuation.DARMActuationModel(self.model_string)
@@ -1500,6 +1512,16 @@ darm_feedback_y = OFF, ON, ON, ON
                 np.angle(actuation_tf[n], deg=True),
                 np.angle(self.known_super_act[n], deg=True))
 
+    def test_stage_super_actuator_drivealign(self):
+        A = pydarm.actuation.DARMActuationModel(self.model_string)
+        actuation_tf = A.stage_super_actuator_drivealign(self.frequencies)
+        for n in range(len(self.frequencies)):
+            self.assertAlmostEqual(np.abs(actuation_tf[n]),
+                                   np.abs(self.known_super_act_drivealign[n]))
+            self.assertAlmostEqual(
+                np.angle(actuation_tf[n], deg=True),
+                np.angle(self.known_super_act_drivealign[n], deg=True))
+
 
 if __name__ == '__main__':
     unittest.main()
