@@ -1,6 +1,8 @@
 import unittest
 import pydarm
 import numpy as np
+import os
+import tempfile
 
 
 class TestOpticalResponseRatio(unittest.TestCase):
@@ -1058,5 +1060,174 @@ foton_delay_filter_tf = test/H1CALCS_8_CLK_DELAY.txt
                 np.angle(self.known_correction[n], deg=True))
 
 
+class TestComputeEpicsRecords(unittest.TestCase):
+
+    known_epics = {
+            'CAL-CS_TDEP_PCAL_LINE1_CORRECTION_REAL': 0.003384106838174459,
+            'CAL-CS_TDEP_PCAL_LINE1_CORRECTION_IMAG': 0.0004044180029064908,
+            'CAL-CS_TDEP_SUS_LINE3_REF_INVA_TST_RESPRATIO_REAL': 1196925220371225.5,
+            'CAL-CS_TDEP_SUS_LINE3_REF_INVA_TST_RESPRATIO_IMAG': -3841998698753872,
+            'CAL-CS_TDEP_SUS_LINE2_REF_INVA_PUM_RESPRATIO_REAL': -4.90204412692278e+17,
+            'CAL-CS_TDEP_SUS_LINE2_REF_INVA_PUM_RESPRATIO_IMAG': -1.9987201437993693e+17,
+            'CAL-CS_TDEP_SUS_LINE1_REF_INVA_UIM_RESPRATIO_REAL': 2.643473241664109e+17,
+            'CAL-CS_TDEP_SUS_LINE1_REF_INVA_UIM_RESPRATIO_IMAG': -1.2695297228124422e+17,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_C_NOCAVPOLE_REAL': 3052033.7355621597,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_C_NOCAVPOLE_IMAG': -1030751.4694586383,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_D_REAL': -243010006.4310854,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_D_IMAG': -1207741923.4087756,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_TST_REAL': -3.6301159301930236e-19,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_TST_IMAG': 5.17252131286287e-19,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_PUM_REAL': -3.486603813748813e-21,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_PUM_IMAG': 2.854144501791321e-21,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_UIM_REAL': 9.669640696236252e-23,
+            'CAL-CS_TDEP_PCAL_LINE2_REF_A_UIM_IMAG': 1.0084874176645285e-22,
+            'CAL-CS_TDEP_PCAL_LINE2_CORRECTION_REAL': 5.919275925544666e-06,
+            'CAL-CS_TDEP_PCAL_LINE2_CORRECTION_IMAG': +3.317567316630787e-07,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_C_NOCAVPOLE_REAL': 3219708.6791652623,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_C_NOCAVPOLE_IMAG': -43662.16666949552,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_D_REAL': 3152795532.787565,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_D_IMAG': 60933868.87924543,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_TST_REAL': -1.1944749080760831e-16,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_TST_IMAG': -2.1720205784065532e-16,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_PUM_REAL': -2.1758869487866404e-16,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_PUM_IMAG': 2.620157641796169e-16,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_UIM_REAL': 4.078008711853952e-17,
+            'CAL-CS_TDEP_PCAL_LINE1_REF_A_UIM_IMAG': 4.992745126138772e-18,
+            'CAL-CS_TDEP_PCAL_LINE3_CORRECTION_REAL': 8.319363469674813e-07,
+            'CAL-CS_TDEP_PCAL_LINE3_CORRECTION_IMAG': 1.1949591405958398e-07,
+            'CAL-CS_TDEP_PCAL_X_COMPARE_CORRECTION_REAL': -3.978067573573798e-06,
+            'CAL-CS_TDEP_PCAL_X_COMPARE_CORRECTION_IMAG': -2.6500091829175134e-07,
+            'CAL-CS_TDEP_PCAL_Y_COMPARE_CORRECTION_REAL': 3.978067573573798e-06,
+            'CAL-CS_TDEP_PCAL_Y_COMPARE_CORRECTION_IMAG': 2.6500091829175134e-07,
+            'CAL-CS_TDEP_SUS_LINE1_SUS_DEMOD_PHASE': -0.3427734375,
+            'CAL-CS_TDEP_SUS_LINE2_SUS_DEMOD_PHASE': -0.36035156249999994,
+            'CAL-CS_TDEP_SUS_LINE3_SUS_DEMOD_PHASE': -0.38671875}
+
+    model_string = '''
+[metadata]
+[interferometer]
+[sensing]
+x_arm_length = 3994.4704
+y_arm_length = 3994.4692
+coupled_cavity_optical_gain = 3.22e6
+coupled_cavity_pole_frequency = 410.6
+detuned_spring_frequency = 4.468
+detuned_spring_Q = 52.14
+sensing_sign = 1
+is_pro_spring = True
+anti_aliasing_rate_string = 16k
+anti_aliasing_method      = biquad
+analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
+omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
+whitening_mode_names = test, test
+omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
+super_high_frequency_poles_apparent_delay = 0, 0
+gain_ratio = 1, 1
+balance_matrix = 1, 1
+omc_path_names = A, B
+single_pole_approximation_delay_correction = -12e-6
+adc_gain = 1, 1
+omc_front_end_trans_amplifier_compensation = ON, ON
+omc_front_end_whitening_compensation_test = ON, ON
+
+[actuation_x_arm]
+darm_feedback_sign = -1
+tst_NpV2 = 4.427e-11
+linearization = OFF
+actuation_esd_bias_voltage = -9.3
+suspension_file = test/H1susdata_O3.mat
+tst_driver_uncompensated_Z_UL = 129.7e3
+tst_driver_uncompensated_Z_LL = 90.74e3
+tst_driver_uncompensated_Z_UR = 93.52e3
+tst_driver_uncompensated_Z_LR = 131.5e3
+tst_driver_uncompensated_P_UL = 3.213e3, 31.5e3
+tst_driver_uncompensated_P_LL = 3.177e3, 26.7e3
+tst_driver_uncompensated_P_UR = 3.279e3, 26.6e3
+tst_driver_uncompensated_P_LR = 3.238e3, 31.6e3
+tst_driver_DC_gain_VpV_HV = 40
+tst_driver_DC_gain_VpV_LV = 1.881
+anti_imaging_rate_string = 16k
+anti_imaging_method = biquad
+analog_anti_imaging_file = test/H1aa.mat
+dac_gain = 7.62939453125e-05
+unknown_actuation_delay = 15e-6
+pum_driver_DC_trans_ApV = 2.6847e-4
+pum_coil_outf_signflip = 1
+pum_NpA = 0.02947
+uim_driver_DC_trans_ApV = 6.1535e-4
+uim_NpA = 1.634
+sus_filter_file = test/H1SUSETMX_1236641144.txt
+tst_isc_inf_bank = ETMX_L3_ISCINF_L
+tst_isc_inf_modules =
+tst_isc_inf_gain = 1.0
+tst_lock_bank = ETMX_L3_LOCK_L
+tst_lock_modules = 5,8,9,10
+tst_lock_gain = 1.0
+tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
+tst_drive_align_modules = 4,5
+tst_drive_align_gain = -35.7
+pum_lock_bank = ETMX_L2_LOCK_L
+pum_lock_modules = 7
+pum_lock_gain = 23.0
+pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
+pum_drive_align_modules = 6,7
+pum_drive_align_gain = 1.0
+uim_lock_bank = ETMX_L1_LOCK_L
+uim_lock_modules = 10
+uim_lock_gain = 1.06
+uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
+uim_drive_align_modules =
+uim_drive_align_gain = 1.0
+
+[actuation]
+darm_output_matrix = 1.0, -1.0, 0.0, 0.0
+darm_feedback_x = OFF, ON, ON, ON
+darm_feedback_y = OFF, OFF, OFF, OFF
+
+[pcal]
+pcal_dewhiten = 1.0, 1.0
+ref_pcal_2_darm_act_sign = -1
+analog_anti_aliasing_file = test/H1aa.mat
+anti_aliasing_rate_string = 16k
+anti_aliasing_method = biquad
+
+[digital]
+digital_filter_file = test/H1OMC_1239468752.txt
+digital_filter_bank = LSC_DARM1, LSC_DARM2
+digital_filter_modules = 1,2,3,4,7,9,10: 3,4,5,6,7
+digital_filter_gain = 400,1
+
+[calcs]
+cal_line_sus_uim_frequency = 15.6
+cal_line_sus_pum_frequency = 16.4
+cal_line_sus_tst_frequency = 17.6
+cal_line_sus_pcal_frequency = 17.1
+cal_line_sens_pcal_frequency = 410.3
+cal_line_high_pcal_frequency = 1083.7
+cal_line_cmp_pcalx_frequency = 500.1
+cal_line_cmp_pcaly_frequency = 500.1
+'''
+
+    def test_compute_epics_records(self):
+        calcs = pydarm.calcs.CALCSModel(self.model_string)
+        test_epics = calcs.compute_epics_records(endstation=True)
+        for key, val in self.known_epics.items():
+            # FIXME: is this enough precision?
+            self.assertAlmostEqual(test_epics[key]/val, 1.0, places=6)
+
+    def test_epics_write_to_file(self):
+        calcs = pydarm.calcs.CALCSModel(self.model_string)
+        test_epics = calcs.compute_epics_records(endstation=True)
+        # FIXME: if we were using pytetst we could simply provide a
+        # `tmp_file` argument to the test method to get a temporary
+        # file from the test harness
+        with tempfile.TemporaryDirectory() as d:
+            save_file = os.path.join(d, 'test_epics_out')
+            pydarm.utils.write_dict_epics(test_epics, dry_run=True, save_file=save_file)
+            with open(save_file) as f:
+                for line in f.readlines():
+                    chan, val = line.strip().split()
+                    self.assertAlmostEqual(self.known_epics[chan]/float(val), 1.0, places=6)
+
 if __name__ == '__main__':
     unittest.main()
