@@ -292,164 +292,6 @@ digital_filter_gain = 400,1
                 np.angle(self.known_response[n], deg=True), 0.0, places=5)
 
 
-class TestComputeEpicsRecords(unittest.TestCase):
-
-    def setUp(self):
-        self.frequencies = np.array(
-            [17.1, 15.6, 16.4, 17.6, 410.3, 1083.7, 500.1, 500.1])
-        self.known_epics = {
-            'CAL-CS_TDEP_PCAL_LINE1_CORRECTION':
-            0.003384106838174459+0.0004044180029064908j,
-            'CAL-CS_TDEP_SUS_LINE3_REF_INVA_TST_RESPRATIO':
-            1196925220371225.5-3841998698753872j,
-            'CAL-CS_TDEP_SUS_LINE2_REF_INVA_PUM_RESPRATIO':
-            -4.90204412692278e+17-1.9987201437993693e+17j,
-            'CAL-CS_TDEP_SUS_LINE1_REF_INVA_UIM_RESPRATIO':
-            2.643473241664109e+17-1.2695297228124422e+17j,
-            'CAL-CS_TDEP_PCAL_LINE2_REF_C_NOCAVPOLE':
-            3052033.7355621597-1030751.4694586383j,
-            'CAL-CS_TDEP_PCAL_LINE2_REF_D':
-            -243010006.4310854-1207741923.4087756j,
-            'CAL-CS_TDEP_PCAL_LINE2_REF_A_TST':
-            -3.6301159301930236e-19+5.17252131286287e-19j,
-            'CAL-CS_TDEP_PCAL_LINE2_REF_A_PUM':
-            -3.486603813748813e-21+2.854144501791321e-21j,
-            'CAL-CS_TDEP_PCAL_LINE2_REF_A_UIM':
-            9.669640696236252e-23+1.0084874176645285e-22j,
-            'CAL-CS_TDEP_PCAL_LINE2_CORRECTION':
-            5.919275925544666e-06+3.317567316630787e-07j,
-            'CAL-CS_TDEP_PCAL_LINE1_REF_C_NOCAVPOLE':
-            3219708.6791652623-43662.16666949552j,
-            'CAL-CS_TDEP_PCAL_LINE1_REF_D':
-            3152795532.787565+60933868.87924543j,
-            'CAL-CS_TDEP_PCAL_LINE1_REF_A_TST':
-            -1.1944749080760831e-16-2.1720205784065532e-16j,
-            'CAL-CS_TDEP_PCAL_LINE1_REF_A_PUM':
-            -2.1758869487866404e-16+2.620157641796169e-16j,
-            'CAL-CS_TDEP_PCAL_LINE1_REF_A_UIM':
-            4.078008711853952e-17+4.992745126138772e-18j,
-            'CAL-CS_TDEP_PCAL_LINE3_CORRECTION':
-            8.319363469674813e-07+1.1949591405958398e-07j,
-            'CAL-CS_TDEP_PCAL_X_COMPARE_CORRECTION':
-            -3.978067573573798e-06-2.6500091829175134e-07j,
-            'CAL-CS_TDEP_PCAL_Y_COMPARE_CORRECTION':
-            3.978067573573798e-06+2.6500091829175134e-07j,
-            'CAL-CS_TDEP_SUS_LINE1_SUS_DEMOD_PHASE': -0.3427734375,
-            'CAL-CS_TDEP_SUS_LINE2_SUS_DEMOD_PHASE': -0.36035156249999994,
-            'CAL-CS_TDEP_SUS_LINE3_SUS_DEMOD_PHASE': -0.38671875}
-
-    def tearDown(self):
-        del self.frequencies
-        del self.known_epics
-
-    def test_compute_epics_records(self):
-        model_string = '''
-[metadata]
-[interferometer]
-[sensing]
-x_arm_length = 3994.4704
-y_arm_length = 3994.4692
-coupled_cavity_optical_gain = 3.22e6
-coupled_cavity_pole_frequency = 410.6
-detuned_spring_frequency = 4.468
-detuned_spring_Q = 52.14
-sensing_sign = 1
-is_pro_spring = True
-anti_aliasing_rate_string = 16k
-anti_aliasing_method      = biquad
-analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
-omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
-whitening_mode_names = test, test
-omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
-super_high_frequency_poles_apparent_delay = 0, 0
-gain_ratio = 1, 1
-balance_matrix = 1, 1
-omc_path_names = A, B
-single_pole_approximation_delay_correction = -12e-6
-adc_gain = 1, 1
-omc_front_end_trans_amplifier_compensation = ON, ON
-omc_front_end_whitening_compensation_test = ON, ON
-
-[actuation_x_arm]
-darm_feedback_sign = -1
-tst_NpV2 = 4.427e-11
-linearization = OFF
-actuation_esd_bias_voltage = -9.3
-suspension_file = test/H1susdata_O3.mat
-tst_driver_uncompensated_Z_UL = 129.7e3
-tst_driver_uncompensated_Z_LL = 90.74e3
-tst_driver_uncompensated_Z_UR = 93.52e3
-tst_driver_uncompensated_Z_LR = 131.5e3
-tst_driver_uncompensated_P_UL = 3.213e3, 31.5e3
-tst_driver_uncompensated_P_LL = 3.177e3, 26.7e3
-tst_driver_uncompensated_P_UR = 3.279e3, 26.6e3
-tst_driver_uncompensated_P_LR = 3.238e3, 31.6e3
-tst_driver_DC_gain_VpV_HV = 40
-tst_driver_DC_gain_VpV_LV = 1.881
-anti_imaging_rate_string = 16k
-anti_imaging_method = biquad
-analog_anti_imaging_file = test/H1aa.mat
-dac_gain = 7.62939453125e-05
-unknown_actuation_delay = 15e-6
-pum_driver_DC_trans_ApV = 2.6847e-4
-pum_coil_outf_signflip = 1
-pum_NpA = 0.02947
-uim_driver_DC_trans_ApV = 6.1535e-4
-uim_NpA = 1.634
-sus_filter_file = test/H1SUSETMX_1236641144.txt
-tst_isc_inf_bank = ETMX_L3_ISCINF_L
-tst_isc_inf_modules =
-tst_isc_inf_gain = 1.0
-tst_lock_bank = ETMX_L3_LOCK_L
-tst_lock_modules = 5,8,9,10
-tst_lock_gain = 1.0
-tst_drive_align_bank = ETMX_L3_DRIVEALIGN_L2L
-tst_drive_align_modules = 4,5
-tst_drive_align_gain = -35.7
-pum_lock_bank = ETMX_L2_LOCK_L
-pum_lock_modules = 7
-pum_lock_gain = 23.0
-pum_drive_align_bank = ETMX_L2_DRIVEALIGN_L2L
-pum_drive_align_modules = 6,7
-pum_drive_align_gain = 1.0
-uim_lock_bank = ETMX_L1_LOCK_L
-uim_lock_modules = 10
-uim_lock_gain = 1.06
-uim_drive_align_bank = ETMX_L1_DRIVEALIGN_L2L
-uim_drive_align_modules =
-uim_drive_align_gain = 1.0
-
-[actuation]
-darm_output_matrix = 1.0, -1.0, 0.0, 0.0
-darm_feedback_x = OFF, ON, ON, ON
-darm_feedback_y = OFF, OFF, OFF, OFF
-
-[pcal]
-pcal_dewhiten = 1.0, 1.0
-ref_pcal_2_darm_act_sign = -1
-analog_anti_aliasing_file = test/H1aa.mat
-anti_aliasing_rate_string = 16k
-anti_aliasing_method = biquad
-
-[digital]
-digital_filter_file = test/H1OMC_1239468752.txt
-digital_filter_bank = LSC_DARM1, LSC_DARM2
-digital_filter_modules = 1,2,3,4,7,9,10: 3,4,5,6,7
-digital_filter_gain = 400,1
-'''
-        darm = pydarm.darm.DARMModel(model_string)
-        test_epics = darm.compute_epics_records(
-            self.frequencies[0], self.frequencies[1], self.frequencies[2],
-            self.frequencies[3], self.frequencies[4], self.frequencies[5],
-            self.frequencies[6], self.frequencies[7], endstation=True)
-        for idx, (key, val) in enumerate(self.known_epics.items()):
-            self.assertAlmostEqual(
-                np.abs(test_epics[key]/self.known_epics[key]), 1.0, places=6)
-            self.assertAlmostEqual(
-                np.angle(test_epics[key], deg=True) -
-                np.angle(self.known_epics[key], deg=True), 0.0, places=5)
-
-
 class TestComputeEtas(unittest.TestCase):
 
     def setUp(self):
