@@ -508,6 +508,66 @@ anti_aliasing_method      = biquad
         self.assertAlmostEqual(std_vals[4] / self.known_std[4], 1.0, places=1)
 
 
+class TestSensingRunMCMCNoDetunedSpring(unittest.TestCase):
+
+    def setUp(self):
+        meas_obj1 = pydarm.measurement.Measurement(
+            './test/2020-01-03_H1_DARM_OLGTF_LF_SS_5to1100Hz_15min.xml')
+        meas_obj2 = pydarm.measurement.Measurement(
+            './test/2020-01-03_H1_PCALY2DARMTF_LF_SS_5t1100Hz_10min.xml')
+
+        model_string = '''
+[metadata]
+[interferometer]
+[sensing]
+x_arm_length = 3994.4704
+y_arm_length = 3994.4692
+sensing_sign = 1
+is_pro_spring = False
+include_spring = False
+anti_aliasing_rate_string = 16k
+anti_aliasing_method      = biquad
+analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
+omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
+whitening_mode_names = test, test
+omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
+super_high_frequency_poles_apparent_delay = 0, 0
+gain_ratio = 1, 1
+balance_matrix = 1, 1
+omc_path_names = A, B
+single_pole_approximation_delay_correction = -12e-6
+adc_gain = 1, 1
+omc_front_end_trans_amplifier_compensation = ON, ON
+omc_front_end_whitening_compensation_test = ON, ON
+[pcal]
+pcal_dewhiten               = 1.0, 1.0
+ref_pcal_2_darm_act_sign    = -1.0
+analog_anti_aliasing_file = test/H1aa.mat
+anti_aliasing_rate_string = 16k
+anti_aliasing_method      = biquad
+'''
+
+        self.meas = pydarm.measurement.ProcessSensingMeasurement(
+            model_string, meas_obj1, meas_obj2,
+            ('H1:LSC-DARM1_IN2', 'H1:LSC-DARM1_EXC'),
+            ('H1:CAL-PCALY_RX_PD_OUT_DQ', 'H1:LSC-DARM_IN1_DQ'),
+            meas1_cohThresh=0.90, meas2_cohThresh=0.992)
+
+    def tearDown(self):
+        del self.meas
+
+    def test_run_mcmc(self):
+        chain = self.meas.run_mcmc(fmin=20, fmax=5000,
+                                   burn_in_steps=100, steps=2000)
+
+        optical_pars = np.median(chain, axis=0)
+        std_vals = np.std(chain, axis=0)
+
+        for n in range(len(optical_pars)):
+            self.assertAlmostEqual(optical_pars[2], 0.0, places=6)
+            self.assertAlmostEqual(optical_pars[3], 1000.0, places=6)
+
+
 class TestInverseSensingFotonFilter(unittest.TestCase):
 
     def setUp(self):
@@ -716,7 +776,7 @@ anti_aliasing_method      = biquad
         self.assertEqual(test[2], self.known_is_pro_spring)
 
 
-class TestSensingStackMeasurements(unittest.TestCase):
+class TestSensingStackMeasurementsGPR(unittest.TestCase):
 
     def setUp(self):
         meas_obj1 = pydarm.measurement.Measurement(
@@ -725,8 +785,6 @@ class TestSensingStackMeasurements(unittest.TestCase):
             './test/2020-01-03_H1_PCALY2DARMTF_LF_SS_5t1100Hz_10min.xml')
 
         model_string = '''
-[metadata]
-[interferometer]
 [sensing]
 x_arm_length = 3994.4704
 y_arm_length = 3994.4692
@@ -768,6 +826,8 @@ anti_aliasing_method      = biquad
             meas1_cohThresh=0.90, meas2_cohThresh=0.992,
             json_results_file='./test/O3_H1_sensing.json')
 
+        self.freq = np.logspace(np.log10(5), np.log10(1200), 20)
+
         self.known_freq = np.array(
             [6.0565, 6.4972, 6.7742, 7.0544, 7.475, 7.773, 8.002, 8.309, 8.791,
              9.103, 10.334, 11.131, 11.709, 12.234, 15.6, 16.4, 17.1, 17.6,
@@ -832,82 +892,53 @@ anti_aliasing_method      = biquad
              0.0053300364, 0.0036761549, 0.003559982, 0.002179605,
              0.0024981925, 0.0041950326, 0.0023519339, 0.0015859917,
              0.0017689841, 0.002548984, 0.0032408999, 0.0077597047])
-
-    def tearDown(self):
-        del self.meas
-        del self.meas2
-        del self.known_freq
-        del self.known_opt_resp
-        del self.known_unc
-
-    def test_stack_measurements(self):
-        test = self.meas.stack_measurements([self.meas2], 20, 5000,
-                                            [20], [5000])
-
-        for n in range(len(test[0][0])):
-            self.assertAlmostEqual(test[0][0][n] / self.known_freq[n], 1.0)
-            self.assertAlmostEqual(
-                np.abs(test[0][1][n])/np.abs(self.known_opt_resp[n]), 1.0, places=6)
-            self.assertAlmostEqual(
-                np.abs(np.angle(test[0][1][n], deg=True) -
-                       np.angle(self.known_opt_resp[n], deg=True)),
-                0.0, places=6)
-            self.assertAlmostEqual(test[0][2][n] / self.known_unc[n], 1.0)
-
-class TestSensingGPR(unittest.TestCase):
-
-    def setUp(self):
-        meas_obj1 = pydarm.measurement.Measurement(
-            './test/2020-01-03_H1_DARM_OLGTF_LF_SS_5to1100Hz_15min.xml')
-        meas_obj2 = pydarm.measurement.Measurement(
-            './test/2020-01-03_H1_PCALY2DARMTF_LF_SS_5t1100Hz_10min.xml')
-
-        model_string = '''
-[metadata]
-[interferometer]
-[sensing]
-x_arm_length = 3994.4704
-y_arm_length = 3994.4692
-sensing_sign = 1
-is_pro_spring = False
-anti_aliasing_rate_string = 16k
-anti_aliasing_method = biquad
-analog_anti_aliasing_file = test/H1aa.mat, test/H1aa.mat
-omc_meas_p_trans_amplifier_uncompensated   = 13.7e3, 17.8e3: 13.7e3, 17.8e3
-whitening_mode_names = test, test
-omc_meas_p_whitening_uncompensated_test   = 11.346e3, 32.875e3, 32.875e3: 11.521e3, 32.863e3, 32.863e3
-super_high_frequency_poles_apparent_delay = 0, 0
-gain_ratio = 1, 1
-balance_matrix = 1, 1
-omc_path_names = A, B
-single_pole_approximation_delay_correction = -12e-6
-adc_gain = 1, 1
-omc_front_end_trans_amplifier_compensation = ON, ON
-omc_front_end_whitening_compensation_test = ON, ON
-[pcal]
-pcal_dewhiten               = 1.0, 1.0
-ref_pcal_2_darm_act_sign    = -1.0
-analog_anti_aliasing_file = test/H1aa.mat
-anti_aliasing_rate_string = 16k
-anti_aliasing_method      = biquad
-'''
-
-        self.meas = pydarm.measurement.ProcessSensingMeasurement(
-            model_string, meas_obj1, meas_obj2,
-            ('H1:LSC-DARM1_IN2', 'H1:LSC-DARM1_EXC'),
-            ('H1:CAL-PCALY_RX_PD_OUT_DQ', 'H1:LSC-DARM_IN1_DQ'),
-            meas1_cohThresh=0.90, meas2_cohThresh=0.992,
-            json_results_file='./test/O3_H1_sensing.json')
-
-        self.meas2 = pydarm.measurement.ProcessSensingMeasurement(
-            model_string, meas_obj1, meas_obj2,
-            ('H1:LSC-DARM1_IN2', 'H1:LSC-DARM1_EXC'),
-            ('H1:CAL-PCALY_RX_PD_OUT_DQ', 'H1:LSC-DARM_IN1_DQ'),
-            meas1_cohThresh=0.90, meas2_cohThresh=0.992,
-            json_results_file='./test/O3_H1_sensing.json')
-
-        self.freq = np.logspace(np.log10(5),np.log10(1200),20)
-
+        self.known_residuals = np.array(
+            [1.5569429604913112-0.2858142213983447j,
+             1.3330399025177055-0.2878013372236475j,
+             1.267307709684956-0.25376513972190556j,
+             1.2383571996494145-0.2103606560242945j,
+             1.166066360831125-0.14523429791806902j,
+             1.1292275953001996-0.17856333844809028j,
+             1.0858664027878329-0.13949487631871102j,
+             1.0744347061098574-0.09681382950262772j,
+             1.0619270162075356-0.0932954754443713j,
+             1.0362094013672696-0.048168864171553534j,
+             1.0416345478011977+0.03775745973067085j,
+             1.0553142021824706+0.003385636447553637j,
+             1.0575427004505358+0.0013736739112262968j,
+             1.038686950975865-0.013707839036249258j,
+             1.0221359671883963+0.013158286123383803j,
+             1.025011291139503+0.0010993604650436912j,
+             1.01295470625067-0.006887542718673806j,
+             1.0116102412954078-0.007072609024523587j,
+             1.0139342444801072-0.004234518260718949j,
+             0.9966608927975363-0.0034841038803181423j,
+             0.9908025332359507-0.0013755965343519622j,
+             0.9876843853332282+0.0035633631709263785j,
+             0.9858417031051571+0.001448396300660967j,
+             0.982619203069668+0.00605873344517217j,
+             0.9870510065774569+0.00845358553385505j,
+             0.9876598912606138+0.011925155956556677j,
+             0.9883587514956639+0.0038846312711781827j,
+             0.9918118718375103+0.00815748349751363j,
+             0.989751684509201+0.008346824672439454j,
+             0.998807352580924+0.009551748762955515j,
+             0.9925369269115982+0.0029944501675775756j,
+             0.9924751348829082+0.004495005016682939j,
+             0.9885703738835263-0.004615179529452968j,
+             0.9959864743479735-0.0008920231444569943j,
+             0.991798202127877+0.0010884312275032872j,
+             0.9865815127807046+0.008180144909614332j,
+             0.9907982850781668+0.008196048128106467j,
+             0.9841644049909182+0.007826507862457676j,
+             0.9919711876100534+0.003214258394364241j,
+             0.9863532169833504+0.005777281890974694j,
+             0.9866095779084851+0.0035041170343225753j,
+             0.984846191385786+0.0021943230636048317j,
+             0.9878400781939896+0.007203056353803067j,
+             0.9859725512522832+0.015387797192340442j])
+        self.known_kappas = np.array(
+            [[1, 408.1817342117136, 0.19227856654546951, 1.6136289842993343]])
         self.known_y_pred = np.array(
              [1.143865894198042-0.13218101017142667j,
               1.1126738395181432-0.10366451924822398j,
@@ -929,7 +960,6 @@ anti_aliasing_method      = biquad
               0.9922985015825642+0.0009075500198081508j,
               0.9904728583448656+0.001664047719329993j,
               0.9872738273572552+0.0034280242769106333j])
-
         self.known_sigma = np.array(
              [0.04027781477576714, 0.030308227084443286, 0.022747613590672393,
               0.017586693851760402, 0.01454809330369326, 0.013007019870077623,
@@ -943,9 +973,38 @@ anti_aliasing_method      = biquad
         del self.meas
         del self.meas2
         del self.freq
+        del self.known_freq
+        del self.known_opt_resp
+        del self.known_unc
+        del self.known_residuals
+        del self.known_kappas
         del self.known_y_pred
         del self.known_sigma
 
+    def test_stack_measurements(self):
+        test, test_kappas = self.meas.stack_measurements(
+            [self.meas2], 20, 5000, [20], [5000])
+
+        for n in range(len(test[0][0])):
+            self.assertAlmostEqual(test[0][0][n] / self.known_freq[n], 1.0)
+            self.assertAlmostEqual(
+                np.abs(test[0][2][n])/np.abs(self.known_opt_resp[n]), 1.0, places=6)
+            self.assertAlmostEqual(
+                np.abs(np.angle(test[0][2][n], deg=True) -
+                       np.angle(self.known_opt_resp[n], deg=True)),
+                0.0, places=6)
+            self.assertAlmostEqual(test[0][3][n] / self.known_unc[n], 1.0)
+            self.assertAlmostEqual(
+                np.abs(test[0][4][n]) / np.abs(self.known_residuals[n]), 1.0)
+            self.assertAlmostEqual(
+                np.angle(test[0][4][n], deg=True) -
+                np.angle(self.known_residuals[n], deg=True),
+                0.0, places=6)
+        for n in range(len(test_kappas)):
+            for m in range(len(test_kappas[n])):
+                self.assertAlmostEqual(
+                    test_kappas[n][m] / self.known_kappas[n][m], 1.0)
+
     def test_sensing_gpr(self):
         test = self.meas.run_gpr(self.freq, [self.meas2], 20, 5000,[20], [5000])
 
@@ -1716,7 +1775,7 @@ anti_aliasing_method      = biquad
         self.assertTrue(np.allclose(test[1], self.known_vals))
 
 
-class TestActuationStackMeasurements(unittest.TestCase):
+class TestActuationStackMeasurementsGPR(unittest.TestCase):
 
     def setUp(self):
         meas_obj1 = pydarm.measurement.Measurement(
@@ -1724,173 +1783,6 @@ class TestActuationStackMeasurements(unittest.TestCase):
         meas_obj2 = pydarm.measurement.Measurement(
             './test/2019-04-24_H1SUSETMX_L3_PCAL2DARM_8min.xml')
         model_string = '''
-[metadata]
-[interferometer]
-[actuation_x_arm]
-darm_feedback_sign = -1
-uim_NpA       = 1.634
-pum_NpA       = 0.02947
-tst_NpV2      = 4.427e-11
-linearization = OFF
-actuation_esd_bias_voltage = -9.3
-sus_filter_file = test/H1SUSETMX_1236641144.txt
-tst_isc_inf_bank    = ETMX_L3_ISCINF_L
-tst_isc_inf_modules =
-tst_isc_inf_gain    = 1.0
-tst_lock_bank       = ETMX_L3_LOCK_L
-tst_lock_modules    = 5,8,9,10
-tst_lock_gain       = 1.0
-tst_drive_align_bank     = ETMX_L3_DRIVEALIGN_L2L
-tst_drive_align_modules  = 4,5
-tst_drive_align_gain     = -35.7
-pum_lock_bank    = ETMX_L2_LOCK_L
-pum_lock_modules = 7
-pum_lock_gain    = 23.0
-pum_drive_align_bank    = ETMX_L2_DRIVEALIGN_L2L
-pum_drive_align_modules = 6,7
-pum_drive_align_gain    = 1.0
-pum_coil_outf_signflip  = 1
-uim_lock_bank    = ETMX_L1_LOCK_L
-uim_lock_modules = 10
-uim_lock_gain    = 1.06
-uim_drive_align_bank    = ETMX_L1_DRIVEALIGN_L2L
-uim_drive_align_modules =
-uim_drive_align_gain    = 1.0
-suspension_file = test/H1susdata_O3.mat
-tst_driver_uncompensated_Z_UL = 129.7e3
-tst_driver_uncompensated_Z_LL = 90.74e3
-tst_driver_uncompensated_Z_UR = 93.52e3
-tst_driver_uncompensated_Z_LR = 131.5e3
-tst_driver_uncompensated_P_UL = 3.213e3, 31.5e3
-tst_driver_uncompensated_P_LL = 3.177e3, 26.7e3
-tst_driver_uncompensated_P_UR = 3.279e3, 26.6e3
-tst_driver_uncompensated_P_LR = 3.238e3, 31.6e3
-tst_front_end_driver_compensation_UL = ON
-tst_front_end_driver_compensation_LL = ON
-tst_front_end_driver_compensation_UR = ON
-tst_front_end_driver_compensation_LR = ON
-pum_front_end_driver_compensation_UL = ON
-pum_front_end_driver_compensation_LL = ON
-pum_front_end_driver_compensation_UR = ON
-pum_front_end_driver_compensation_LR = ON
-uim_front_end_driver_compensation_UL = ON
-uim_front_end_driver_compensation_LL = ON
-uim_front_end_driver_compensation_UR = ON
-uim_front_end_driver_compensation_LR = ON
-tst_driver_DC_gain_VpV_HV = 40
-tst_driver_DC_gain_VpV_LV = 1.881
-pum_driver_DC_trans_ApV = 2.6847e-4
-uim_driver_DC_trans_ApV = 6.1535e-4
-anti_imaging_rate_string = 16k
-anti_imaging_method      = biquad
-analog_anti_imaging_file = test/H1aa.mat
-dac_gain = 7.62939453125e-05
-unknown_actuation_delay = 15e-6
-[pcal]
-pcal_dewhiten               = 1.0, 1.0
-ref_pcal_2_darm_act_sign    = -1.0
-analog_anti_aliasing_file = test/H1aa.mat
-anti_aliasing_rate_string = 16k
-anti_aliasing_method      = biquad
-'''
-
-        self.meas = pydarm.measurement.ProcessActuationMeasurement(
-            model_string, 'actuation_x_arm', meas_obj1, meas_obj2,
-            ('H1:SUS-ETMX_L3_CAL_EXC', 'H1:LSC-DARM_IN1_DQ'),
-            ('H1:CAL-PCALY_RX_PD_OUT_DQ', 'H1:LSC-DARM_IN1_DQ'),
-            meas1_cohThresh=0.9, meas2_cohThresh=0.9,
-            json_results_file='./test/O3_H1_actuation.json')
-
-        self.meas2 = pydarm.measurement.ProcessActuationMeasurement(
-            model_string, 'actuation_x_arm', meas_obj1, meas_obj2,
-            ('H1:SUS-ETMX_L3_CAL_EXC', 'H1:LSC-DARM_IN1_DQ'),
-            ('H1:CAL-PCALY_RX_PD_OUT_DQ', 'H1:LSC-DARM_IN1_DQ'),
-            meas1_cohThresh=0.9, meas2_cohThresh=0.9,
-            json_results_file='./test/O3_H1_actuation.json')
-
-        self.known_freq = np.array(
-            [7.0, 7.6529, 8.3666, 10.0, 12.1153, 14.678, 15.6, 16.4, 17.1,
-             17.6, 19.1, 21.5443, 26.1016, 31.6228, 38.3119, 46.4159, 56.2341,
-             68.1292, 82.54, 100.0, 134.224, 177.828, 237.137, 292.211,
-             340.654, 433.7, 490.327, 543.213, 625.742, 699.465, 749.894,
-             824.265, 962.764, 1221.57])
-        self.known_act_resp = np.array(
-            [4.600713959860163e-11+1.0080589647955395e-12j,
-             4.3249832259388226e-11+4.408071320942729e-13j,
-             4.48155795609468e-11+3.86874225385866e-13j,
-             4.4660708979444904e-11-3.8004875444982484e-13j,
-             4.425263927858843e-11-3.5425549114772793e-14j,
-             4.421476396126438e-11-1.4203840636546456e-13j,
-             4.421522955625061e-11-1.2553594713881172e-13j,
-             4.418636660252522e-11-1.0700918520635529e-13j,
-             4.4187661466837905e-11-7.898033591443161e-14j,
-             4.426465311599576e-11-5.757913893767196e-14j,
-             4.426198835523701e-11-4.836467037182701e-14j,
-             4.4212504188651004e-11-9.98667602352944e-14j,
-             4.417750488110072e-11-6.587983024452382e-14j,
-             4.4171939096159955e-11-5.183099895588114e-14j,
-             4.4144233164685666e-11-8.0896350040868e-14j,
-             4.4165781727314084e-11-8.928868719903062e-14j,
-             4.420975466340609e-11-1.0683151491413745e-13j,
-             4.413865448552216e-11-1.2087638132471638e-13j,
-             4.415523144790698e-11-2.1795609414185902e-13j,
-             4.4083317409918716e-11-2.139793293142482e-13j,
-             4.401117360813622e-11-2.57927709143044e-13j,
-             4.416408137947615e-11-4.1265399688173483e-13j,
-             4.435259473093628e-11-2.2381956444257016e-13j,
-             4.366185379385905e-11-7.636270168782189e-13j,
-             4.438924517531857e-11-9.074260365559163e-13j,
-             4.383262308491154e-11-6.937746864860927e-13j,
-             4.3839230496539004e-11-6.027727906294568e-13j,
-             4.417041964253939e-11-1.3423056309147865e-12j,
-             4.3744330940010925e-11-1.4039314062379411e-12j,
-             4.4228525304498916e-11-5.450496421816152e-13j,
-             4.43374152373744e-11-7.487397304199775e-13j,
-             4.210386132796535e-11-3.4919213295478316e-13j,
-             4.3893120670781536e-11-1.5316483233478304e-12j,
-             4.4104164987097186e-11-1.113668550447501e-12j])
-        self.known_unc = np.array(
-            [0.04250459, 0.012876861, 0.010742824, 0.007582212, 0.0015632747,
-             0.0009036516, 0.0010863609, 0.0009886964, 0.00071596145,
-             0.0013702465, 0.0005567265, 0.00044350346, 0.00042286396,
-             0.00064130547, 0.0005777424, 0.00079860573, 0.0009036517,
-             0.0010697736, 0.0007159614, 0.0017246293, 0.0041964506,
-             0.0033547822, 0.005654715, 0.008364857, 0.0037388739,
-             0.0041864677, 0.003776119, 0.0048317686, 0.0060286755,
-             0.0065562637, 0.0060102274, 0.0333514, 0.014210214, 0.023705741])
-
-    def tearDown(self):
-        del self.meas
-        del self.meas2
-        del self.known_freq
-        del self.known_act_resp
-        del self.known_unc
-
-    def test_stack_measurements(self):
-        test = self.meas.stack_measurements([self.meas2], 20, 600,
-                                            [20], [600])
-
-        for n in range(len(test[0][0])):
-            self.assertAlmostEqual(test[0][0][n] / self.known_freq[n], 1.0,
-                                   places=6)
-            self.assertAlmostEqual(
-                np.abs(test[0][1][n])/np.abs(self.known_act_resp[n]), 1.0)
-            self.assertAlmostEqual(
-                np.abs(np.angle(test[0][1][n], deg=True) -
-                       np.angle(self.known_act_resp[n], deg=True)),
-                0.0, places=6)
-            self.assertAlmostEqual(test[0][2][n] / self.known_unc[n], 1.0)
-
-class TestActuationGPR(unittest.TestCase):
-
-    def setUp(self):
-        meas_obj1 = pydarm.measurement.Measurement(
-            './test/2019-04-24_H1SUSETMX_L3_iEXC2DARM_8min.xml')
-        meas_obj2 = pydarm.measurement.Measurement(
-            './test/2019-04-24_H1SUSETMX_L3_PCAL2DARM_8min.xml')
-        model_string = '''
-[metadata]
-[interferometer]
 [actuation_x_arm]
 darm_feedback_sign = -1
 uim_NpA       = 1.634
@@ -1975,6 +1867,92 @@ anti_aliasing_method      = biquad
 
         self.freq = np.logspace(np.log10(5),np.log10(500),20)
 
+        self.known_freq = np.array(
+            [7.0, 7.6529, 8.3666, 10.0, 12.1153, 14.678, 15.6, 16.4, 17.1,
+             17.6, 19.1, 21.5443, 26.1016, 31.6228, 38.3119, 46.4159, 56.2341,
+             68.1292, 82.54, 100.0, 134.224, 177.828, 237.137, 292.211,
+             340.654, 433.7, 490.327, 543.213, 625.742, 699.465, 749.894,
+             824.265, 962.764, 1221.57])
+        self.known_act_resp = np.array(
+            [4.600713959860163e-11+1.0080589647955395e-12j,
+             4.3249832259388226e-11+4.408071320942729e-13j,
+             4.48155795609468e-11+3.86874225385866e-13j,
+             4.4660708979444904e-11-3.8004875444982484e-13j,
+             4.425263927858843e-11-3.5425549114772793e-14j,
+             4.421476396126438e-11-1.4203840636546456e-13j,
+             4.421522955625061e-11-1.2553594713881172e-13j,
+             4.418636660252522e-11-1.0700918520635529e-13j,
+             4.4187661466837905e-11-7.898033591443161e-14j,
+             4.426465311599576e-11-5.757913893767196e-14j,
+             4.426198835523701e-11-4.836467037182701e-14j,
+             4.4212504188651004e-11-9.98667602352944e-14j,
+             4.417750488110072e-11-6.587983024452382e-14j,
+             4.4171939096159955e-11-5.183099895588114e-14j,
+             4.4144233164685666e-11-8.0896350040868e-14j,
+             4.4165781727314084e-11-8.928868719903062e-14j,
+             4.420975466340609e-11-1.0683151491413745e-13j,
+             4.413865448552216e-11-1.2087638132471638e-13j,
+             4.415523144790698e-11-2.1795609414185902e-13j,
+             4.4083317409918716e-11-2.139793293142482e-13j,
+             4.401117360813622e-11-2.57927709143044e-13j,
+             4.416408137947615e-11-4.1265399688173483e-13j,
+             4.435259473093628e-11-2.2381956444257016e-13j,
+             4.366185379385905e-11-7.636270168782189e-13j,
+             4.438924517531857e-11-9.074260365559163e-13j,
+             4.383262308491154e-11-6.937746864860927e-13j,
+             4.3839230496539004e-11-6.027727906294568e-13j,
+             4.417041964253939e-11-1.3423056309147865e-12j,
+             4.3744330940010925e-11-1.4039314062379411e-12j,
+             4.4228525304498916e-11-5.450496421816152e-13j,
+             4.43374152373744e-11-7.487397304199775e-13j,
+             4.210386132796535e-11-3.4919213295478316e-13j,
+             4.3893120670781536e-11-1.5316483233478304e-12j,
+             4.4104164987097186e-11-1.113668550447501e-12j])
+        self.known_unc = np.array(
+            [0.04250459, 0.012876861, 0.010742824, 0.007582212, 0.0015632747,
+             0.0009036516, 0.0010863609, 0.0009886964, 0.00071596145,
+             0.0013702465, 0.0005567265, 0.00044350346, 0.00042286396,
+             0.00064130547, 0.0005777424, 0.00079860573, 0.0009036517,
+             0.0010697736, 0.0007159614, 0.0017246293, 0.0041964506,
+             0.0033547822, 0.005654715, 0.008364857, 0.0037388739,
+             0.0041864677, 0.003776119, 0.0048317686, 0.0060286755,
+             0.0065562637, 0.0060102274, 0.0333514, 0.014210214, 0.023705741])
+        self.known_residuals = np.array(
+            [1.0414813601161887+0.022819819516257986j,
+             0.9790631306237049+0.009978721036640813j,
+             1.014507556066534+0.008757821028973616j,
+             1.0110016909960398-0.008603310204271913j,
+             1.0017640598222484-0.0008019418145058349j,
+             1.000906661660111-0.003215378176283788j,
+             1.000917201513684-0.002841805643333792j,
+             1.0002638197004539-0.0024224081893841364j,
+             1.0002931320433939-0.0017879083197032627j,
+             1.0020360216946125-0.0013034411711702527j,
+             1.0019756984780221-0.0010948496931299358j,
+             1.0008555063174964-0.002260722361467221j,
+             1.0000632134965757-0.001491347121509235j,
+             0.9999372186768527-0.0011733183102248664j,
+             0.9993100288224103-0.0018312818707686433j,
+             0.9997978319441928-0.0020212624432164174j,
+             1.0007932642552115-0.002418386198964461j,
+             0.9991837421112283-0.0027363252557993595j,
+             0.9995590012011027-0.004933956150215548j,
+             0.997931055392071-0.004843932591306733j,
+             0.9962979083314318-0.005838809012634388j,
+             0.9997593405147903-0.009341407652936241j,
+             1.0040267899453583-0.005066689788443787j,
+             0.9883902209927521-0.017286519247024575j,
+             1.004856460187804-0.020541753106512746j,
+             0.9922559912854909-0.015705245107838658j,
+             0.9924055658104199-0.013645200099644404j,
+             0.9999028222198985-0.030386290180061646j,
+             0.990257287954611-0.03178133661986767j,
+             1.0012181824960844-0.012338498929323191j,
+             1.0036831658961685-0.01694951000270601j,
+             0.9531213447572185-0.007904796967381247j,
+             0.9936254984656036-0.03467251372202799j,
+             0.9984029900358723-0.02521054442365389j])
+        self.known_kappas = [1.0]
         self.known_y_pred = np.array(
              [1.0006795 -0.00373967j,
               1.0010006 -0.00311973j,
@@ -1995,21 +1973,48 @@ anti_aliasing_method      = biquad
               0.9977593 -0.01195099j,
               0.99733252-0.01402785j,
               0.99683654-0.01620487j,
-              0.99625362-0.0184465j ])
-
+              0.99625362-0.0184465j])
         self.known_sigma = np.array(
              [0.04130269, 0.03193534, 0.02395265, 0.0174245, 0.0124513, 0.00919158,
               0.0077308, 0.00769416, 0.00836436, 0.00926477, 0.01023673, 0.01125869,
               0.01232628, 0.01341221, 0.01447537, 0.01549535, 0.01651721, 0.01769553,
-              0.01931754, 0.0217703 ])
+              0.01931754, 0.0217703])
 
     def tearDown(self):
         del self.meas
         del self.meas2
         del self.freq
+        del self.known_freq
+        del self.known_act_resp
+        del self.known_unc
+        del self.known_residuals
+        del self.known_kappas
         del self.known_y_pred
         del self.known_sigma
 
+    def test_stack_measurements(self):
+        test, test_kappas = self.meas.stack_measurements([self.meas2], 20, 600,
+                                            [20], [600])
+
+        for n in range(len(test[0][0])):
+            self.assertAlmostEqual(test[0][0][n] / self.known_freq[n], 1.0,
+                                   places=6)
+            self.assertAlmostEqual(
+                np.abs(test[0][2][n])/np.abs(self.known_act_resp[n]), 1.0)
+            self.assertAlmostEqual(
+                np.abs(np.angle(test[0][2][n], deg=True) -
+                       np.angle(self.known_act_resp[n], deg=True)),
+                0.0, places=6)
+            self.assertAlmostEqual(test[0][3][n] / self.known_unc[n], 1.0)
+            self.assertAlmostEqual(
+                np.abs(test[0][4][n])/np.abs(self.known_residuals[n]), 1.0)
+            self.assertAlmostEqual(
+                np.angle(test[0][4][n], deg=True) -
+                np.angle(self.known_residuals[n], deg=True),
+                0.0, places=6)
+        for n in range(len(test_kappas)):
+            self.assertAlmostEqual(test_kappas[n] / self.known_kappas[n], 1.0)
+
     def test_actuation_gpr(self):
         test = self.meas.run_gpr(self.freq, [self.meas2], 20, 600,[20], [600])
 
