@@ -198,15 +198,6 @@ uim_mcmc =
 tst_gpr =
 pum_gpr =
 uim_gpr =
-[x-arm-sus-cal-lines]
-uim = 15.6
-pum = 16.4
-tst = 17.6
-[y-arm-pcal-cal-lines]
-sus = 17.1
-sensing = 410.3
-pcal3 = 1083.7
-pcal4 = 7.93
 [tdcf-data]
 frametype = R
 duration = 130
@@ -259,15 +250,6 @@ uim_mcmc =
 tst_gpr =
 pum_gpr =
 uim_gpr =
-[x-arm-sus-cal-lines]
-uim = 15.6
-pum = 16.4
-tst = 17.6
-[y-arm-pcal-cal-lines]
-sus = 17.1
-sensing = 410.3
-pcal3 = 1083.7
-pcal4 = 7.93
 [tdcf-data]
 frametype = R
 duration = 130
@@ -352,15 +334,6 @@ uim_mcmc =
 tst_gpr =
 pum_gpr =
 uim_gpr =
-[x-arm-sus-cal-lines]
-uim = 15.6
-pum = 16.4
-tst = 17.6
-[y-arm-pcal-cal-lines]
-sus = 17.1
-sensing = 410.3
-pcal3 = 1083.7
-pcal4 = 7.93
 [tdcf-data]
 frametype = R
 duration = 130
@@ -416,15 +389,6 @@ uim_mcmc =
 tst_gpr = 
 pum_gpr = 
 uim_gpr = 
-[x-arm-sus-cal-lines]
-uim = 15.6
-pum = 16.4
-tst = 17.6
-[y-arm-pcal-cal-lines]
-sus = 17.1
-sensing = 410.3
-pcal3 = 1083.7
-pcal4 = 7.93
 [tdcf-data]
 frametype = R
 duration = 130
@@ -475,7 +439,7 @@ sample = False'''
             'example_model_files/H1_20190416.ini', self.config)
         
         samples = test_unc.compute_response_uncertainty(
-            1239958818, self.frequencies, trials=1, data=self.data)
+            1239958818, self.frequencies, trials=50, data=self.data)
 
         for n in range(len(self.frequencies)):
             self.assertAlmostEqual(
@@ -501,6 +465,86 @@ sample = False'''
                 np.angle(samples[0][n], deg=True) -
                 np.angle(self.known_ratio[n], deg=True), 0.0, places=5)
 
+class TestComputeConvolvedResponseUncertainty(unittest.TestCase):
+
+    def setUp(self):
+        self.data = tsd.read('./test/test_ch_data.hdf5')
+        self.frequencies = np.logspace(0, np.log10(5000.), 10)
+        self.config = '''[reference-model]
+model =
+previous_model_response_curve_file = test/test_response_curves.hdf5
+[sensing-measurement-files]
+mcmc = 
+gpr = 
+[x-arm-measurement-files]
+tst_mcmc = 
+pum_mcmc = 
+uim_mcmc = 
+tst_gpr = 
+pum_gpr = 
+uim_gpr = 
+[tdcf-data]
+frametype = R
+duration = 130
+[sensing-tdcf]
+kappa_c = CAL-CS_TDEP_KAPPA_C_OUTPUT
+f_cc = CAL-CS_TDEP_F_C_OUTPUT
+pcal2_unc = CAL-CS_TDEP_PCAL_LINE2_UNCERTAINTY
+pcal_arm = Y
+[x-arm-tdcf]
+kappa_uim = CAL-CS_TDEP_KAPPA_UIM_REAL_OUTPUT
+kappa_pum = CAL-CS_TDEP_KAPPA_PUM_REAL_OUTPUT
+kappa_tst = CAL-CS_TDEP_KAPPA_TST_REAL_OUTPUT
+pcal1_unc = CAL-CS_TDEP_PCAL_LINE1_UNCERTAINTY
+uim_unc = CAL-CS_TDEP_SUS_LINE1_UNCERTAINTY
+pum_unc = CAL-CS_TDEP_SUS_LINE2_UNCERTAINTY
+tst_unc = CAL-CS_TDEP_SUS_LINE3_UNCERTAINTY
+pcal_arm = Y
+[sample-tdcf]
+kappa_c = False
+f_cc = False
+kappa_uim = False
+kappa_pum = False
+kappa_tst = False
+[hoft-tdcf-data-application]
+kappa_c = True
+f_cc = True
+kappa_uim = True
+kappa_pum = True
+kappa_tst = True
+pcal_sys_err = False
+[pcal]
+sys_err =
+sys_unc = 0.01
+sample = True'''
+
+        os.environ['CAL_DATA_ROOT'] = './test'
+
+    def tearDown(self):
+        del self.data
+        del self.frequencies
+        del os.environ['CAL_DATA_ROOT']
+        del self.config
+        
+
+    def test_compute_response_uncertainty_convolution(self):
+        test_unc = pydarm.uncertainty.DARMUncertainty(
+            'example_model_files/H1_20190416.ini', self.config)
+
+        samples = test_unc.compute_response_uncertainty(
+            1239958818, self.frequencies, trials=5, data=self.data,
+            seed=1234)
+
+        _, old_samples = pydarm.utils.read_response_curves_from_hdf5(
+            test_unc.response_curve_file)
+
+        for n in range(len(self.frequencies)):
+            self.assertAlmostEqual(
+                np.abs(samples[0][n]) / np.abs(old_samples[0][n])**2, 1.0,
+                places=6)
+            self.assertAlmostEqual(
+                np.angle(samples[0][n], deg=True) -
+                2 * np.angle(old_samples[0][n], deg=True), 0.0, places=5)
 
 class TestResponseQuantiles(unittest.TestCase):
 
@@ -533,6 +577,7 @@ class TestPlotResponseSamples(unittest.TestCase):
         self.response_mag_quant, self.response_pha_quant = \
           pydarm.uncertainty.DARMUncertainty.response_quantiles(
               self.response_samples)
+        self.ifo = 'LHO'
 
     def tearDown(self):
         del self.response_samples
@@ -541,8 +586,9 @@ class TestPlotResponseSamples(unittest.TestCase):
         del self.RRPha
         del self.response_mag_quant
         del self.response_pha_quant
+        del self.ifo
 
     def test_plot_response_samples(self):
         pydarm.uncertainty.DARMUncertainty.plot_response_samples(
             self.freq, self.RRMag, self.RRPha, self.response_mag_quant,
-            self.response_pha_quant)
+            self.response_pha_quant, self.ifo)
\ No newline at end of file
