@@ -9,7 +9,8 @@ from pydarm.utils import (serielZPK, parallelZPK,
                           read_chain_from_hdf5,
                           save_gpr_to_hdf5,
                           read_gpr_from_hdf5,
-                          read_eta_or_syserr_from_hdf5)
+                          read_eta_or_syserr_from_hdf5,
+                          thiran_delay_filter)
 
 
 class TestSerielZPK(unittest.TestCase):
@@ -382,5 +383,27 @@ class TestReadEtaOrSysErrFromHDF5(unittest.TestCase):
                 np.angle(self.known_eta[n], deg=True), 0.0, places=4)
 
 
+class TestThiranDelayFilter(unittest.TestCase):
+
+    def setUp(self):
+        self.tau = 2.5
+        self.Ts = 0.5
+        self.filter = signal.TransferFunction([0, 0, 0, 0, 0, 1], [1, 0, 0, 0, 0, 0], dt=self.Ts)
+
+    def tearDown(self):
+        del self.tau
+        del self.Ts
+        del self.filter
+
+    def test_thiran_delay_filter(self):
+        test_filter = thiran_delay_filter(self.tau, self.Ts)
+        self.assertEqual(len(test_filter.num), len(self.filter.num))
+        self.assertEqual(len(test_filter.den), len(self.filter.den))
+        self.assertEqual(test_filter.dt, self.filter.dt)
+        for n in range(len(test_filter.num)):
+            self.assertAlmostEqual(test_filter.num[n], self.filter.num[n])
+            self.assertAlmostEqual(test_filter.den[n], self.filter.den[n])
+
+
 if __name__ == '__main__':
     unittest.main()
